@@ -17,16 +17,29 @@
  * along with Zrythm.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-#ifndef __AUDIO_PORTS_H__
-#define __AUDIO_PORTS_H__
-
-/** \file
+/**
+ * \file
+ *
  * Ports that transfer audio/midi/other signals to
  * one another.
  */
 
+#ifndef __AUDIO_PORTS_H__
+#define __AUDIO_PORTS_H__
+
 #include "utils/yaml.h"
 
+typedef struct Plugin Plugin;
+typedef struct MidiEvents MidiEvents;
+typedef struct Fader Fader;
+typedef struct SampleProcessor SampleProcessor;
+
+/**
+ * @addtogroup audio
+ *
+ * @{
+ */
+
 #define MAX_DESTINATIONS 600
 #define FOREACH_SRCS(port) \
   for (int i = 0; i < port->num_srcs; i++)
@@ -39,9 +52,6 @@
  */
 #define PORT_NOT_OWNED -1
 
-typedef struct Plugin Plugin;
-typedef struct MidiEvents MidiEvents;
-
 /**
  * Direction of the signal.
  */
@@ -70,8 +80,13 @@ typedef enum PortOwnerType
   PORT_OWNER_TYPE_BACKEND,
   PORT_OWNER_TYPE_PLUGIN,
   PORT_OWNER_TYPE_TRACK,
+  PORT_OWNER_TYPE_FADER,
+  PORT_OWNER_TYPE_SAMPLE_PROCESSOR,
 } PortOwnerType;
 
+/**
+ * Port flags.
+ */
 typedef enum PortFlags
 {
   PORT_FLAG_STEREO_L = 0x01,
@@ -112,6 +127,7 @@ typedef enum PortInternalType
 typedef struct Lv2Port Lv2Port;
 typedef struct Channel Channel;
 typedef struct Track Track;
+typedef struct SampleProcessor SampleProcessor;
 typedef enum PanAlgorithm PanAlgorithm;
 typedef enum PanLaw PanLaw;
 
@@ -207,6 +223,7 @@ typedef struct Port
 
   Plugin              * plugin;
   Track             * track;
+  SampleProcessor * sample_processor;
 
   /** used when loading projects FIXME needed? */
   int                 initialized;
@@ -228,6 +245,24 @@ typedef struct Port
    */
   //int                 has_modulators;
 
+  /**
+   * Capture latency.
+   *
+   * See page 116 of "The Ardour DAW - Latency
+   * Compensation and Anywhere-to-Anywhere Signal
+   * Routing Systems".
+   */
+  long                capture_latency;
+
+  /**
+   * Playback latency.
+   *
+   * See page 116 of "The Ardour DAW - Latency
+   * Compensation and Anywhere-to-Anywhere Signal
+   * Routing Systems".
+   */
+  long                playback_latency;
+
   /** Port undergoing deletion. */
   int                 deleting;
 } Port;
@@ -532,30 +567,58 @@ port_disconnect_dests (Port * src)
 
 /**
  * Apply given fader value to port.
+ *
+ * @param start_frame The start frame offset from
+ *   0 in this cycle.
+ * @param nframes The number of frames to process.
  */
 void
-port_apply_fader (Port * port, float amp);
+port_apply_fader (
+  Port *    port,
+  float     amp,
+  const int start_frame,
+  const int nframes);
 
 /**
- * First sets port buf to 0, then sums the given port signal from its inputs.
+ * First sets port buf to 0, then sums the given
+ * port signal from its inputs.
+ *
+ * @param start_frame The start frame offset from
+ *   0 in this cycle.
+ * @param nframes The number of frames to process.
+ * @param noroll Clear the port buffer in this
+ *   range.
  */
 void
-port_sum_signal_from_inputs (Port * port);
+port_sum_signal_from_inputs (
+  Port *    port,
+  const int start_frame,
+  const int nframes,
+  const int noroll);
 
 /**
- * Sets the owner channel & its ID.
+ * Sets the owner track & its ID.
  */
-static inline void
+void
 port_set_owner_track (
   Port *    port,
-  Track *   track)
-{
-  g_warn_if_fail (port && track);
+  Track *   track);
 
-  port->track = track;
-  port->identifier.owner_type =
-    PORT_OWNER_TYPE_TRACK;
-}
+/**
+ * Sets the owner sample processor.
+ */
+void
+port_set_owner_sample_processor (
+  Port *   port,
+  SampleProcessor * sample_processor);
+
+/**
+ * Sets the owner fader & its ID.
+ */
+void
+port_set_owner_fader (
+  Port *    port,
+  Fader *   fader);
 
 /**
  * Sets the owner plugin & its ID.
@@ -646,13 +709,23 @@ port_apply_pan_stereo (Port *       l,
                        PanAlgorithm pan_algo);
 
 /**
- * Applies the pan to the given L/R ports.
+ * Applies the pan to the given port.
+ *
+ * @param start_frame The start frame offset from
+ *   0 in this cycle.
+ * @param nframes The number of frames to process.
  */
 void
 port_apply_pan (
-  Port *       self,
+  Port *       port,
   float        pan,
   PanLaw       pan_law,
-  PanAlgorithm pan_algo);
+  PanAlgorithm pan_algo,
+  const int    start_frame,
+  const int    nframes);
+
+/**
+ * @}
+ */
 
 #endif
