@@ -20,8 +20,7 @@
 /**
  * \file
  *
- * The transport struct containing all objects in
- *  the transport and their metadata
+ * Transport API.
  */
 
 #ifndef __AUDIO_TRANSPORT_H__
@@ -46,9 +45,11 @@
 #define DEFAULT_BEATS_PER_BAR 4
 #define DEFAULT_BEAT_UNIT BEAT_UNIT_4
 
-#define PLAYHEAD TRANSPORT->playhead_pos
+#define PLAYHEAD (&TRANSPORT->playhead_pos)
 #define IS_TRANSPORT_ROLLING \
-  TRANSPORT->play_state == PLAYSTATE_ROLLING
+  (TRANSPORT->play_state == PLAYSTATE_ROLLING)
+#define IS_TRANSPORT_LOOPING \
+  (TRANSPORT->loop)
 
 typedef enum BeatUnit
 {
@@ -131,6 +132,9 @@ typedef struct Transport
   /** Recording or not. */
   int               recording;
 
+  /** Metronome enabled or not. */
+  int               metronome_enabled;
+
   /**
    * This is set when record is toggled and is used to check
    * if a new region should be created.
@@ -240,10 +244,22 @@ transport_set_beat_unit (
   int beat_unit);
 
 /**
- * Moves the playhead by the time corresponding to given samples.
+ * Sets whether metronome is enabled or not.
  */
 void
-transport_add_to_playhead (int nframes);
+transport_set_metronome_enabled (
+  Transport * self,
+  const int   enabled);
+
+/**
+ * Moves the playhead by the time corresponding to
+ * given samples, taking into account the loop
+ * end point.
+ */
+void
+transport_add_to_playhead (
+  Transport * self,
+  const int   nframes);
 
 void
 transport_request_pause ();
@@ -309,9 +325,38 @@ transport_goto_next_marker (
 void
 transport_update_position_frames ();
 
+/**
+ * Adds frames to the given global frames, while
+ * adjusting the new frames to loop back if the
+ * loop point was crossed.
+ *
+ * @return The new frames adjusted.
+ */
+long
+transport_frames_add_frames (
+  const Transport * self,
+  const long        gframes,
+  const int         frames);
+
+/**
+ * Adds frames to the given position similar to
+ * position_add_frames, except that it adjusts
+ * the new Position if the loop end point was
+ * crossed.
+ */
+void
+transport_position_add_frames (
+  const Transport * self,
+  Position *        pos,
+  const int         frames);
+
 void
 transport_set_ebeat_unit (
   Transport * self,
   BeatUnit bu);
 
+/**
+ * @}
+ */
+
 #endif
