@@ -25,9 +25,6 @@
 
 #include "config.h"
 
-#include "audio/instrument_track.h"
-#include "audio/track.h"
-#include "audio/transport.h"
 #include "actions/actions.h"
 #include "actions/undo_manager.h"
 #include "actions/create_tracks_action.h"
@@ -35,6 +32,10 @@
 #include "actions/delete_midi_arranger_selections_action.h"
 #include "actions/delete_timeline_selections_action.h"
 #include "actions/duplicate_midi_arranger_selections_action.h"
+#include "actions/quantize_timeline_selections.h"
+#include "audio/instrument_track.h"
+#include "audio/track.h"
+#include "audio/transport.h"
 #include "gui/backend/midi_arranger_selections.h"
 #include "gui/backend/timeline_selections.h"
 #include "gui/backend/tracklist_selections.h"
@@ -44,15 +45,17 @@
 #include "gui/widgets/center_dock.h"
 #include "gui/widgets/center_dock_bot_box.h"
 #include "gui/widgets/clip_editor.h"
+#include "gui/widgets/clip_editor_inner.h"
 #include "gui/widgets/donate_dialog.h"
 #include "gui/widgets/export_dialog.h"
 #include "gui/widgets/main_window.h"
 #include "gui/widgets/midi_arranger.h"
 #include "gui/widgets/midi_modifier_arranger.h"
-#include "gui/widgets/midi_ruler.h"
+#include "gui/widgets/editor_ruler.h"
+#include "gui/widgets/midi_editor_space.h"
 #include "gui/widgets/mixer.h"
-#include "gui/widgets/piano_roll.h"
 #include "gui/widgets/preferences.h"
+#include "gui/widgets/quantize_dialog.h"
 #include "gui/widgets/ruler.h"
 #include "gui/widgets/timeline_arranger.h"
 #include "gui/widgets/timeline_bg.h"
@@ -66,6 +69,7 @@
 #include "utils/gtk.h"
 #include "utils/io.h"
 #include "utils/resources.h"
+#include "utils/string.h"
 
 #include <gtk/gtk.h>
 
@@ -222,16 +226,45 @@ activate_about (GSimpleAction *action,
     GTK_IMAGE (
       resources_get_icon (ICON_TYPE_ZRYTHM,
                           "z-splash.svg"));
+
+  char * artists[] =
+    {
+      "Alexandros Theodotou <alex@zrythm.org>",
+      NULL
+    };
+  char * authors[] =
+    {
+      "Alexandros Theodotou <alex@zrythm.org>",
+      "Sascha Bast <sash@mischkonsum.org>",
+      "Georg Krause",
+      NULL
+    };
+  char * documenters[] =
+    {
+      "Alexandros Theodotou <alex@zrythm.org>",
+      NULL
+    };
+  char * translators =
+      "Alexandros Theodotou <alex@zrythm.org>\n"
+      "Nicolas Faure <sub26nico@laposte.net>\n"
+      "Waui <wau-wau@tutanota.de>\n"
+      "Allan Nordhøy <epost@anotheragency.no>\n"
+      "WaldiS <admin@sto.ugu.pl>";
+
   gtk_show_about_dialog (
     GTK_WINDOW (MAIN_WINDOW),
+    "artists", artists,
+    "authors", authors,
     "copyright", "Copyright (C) 2018-2019 Alexandros Theodotou",
+    "documenters", documenters,
     /*"logo-icon-name", "z",*/
     "logo", gtk_image_get_pixbuf (img),
     "program-name", "Zrythm",
     "comments", "An highly automated, intuitive, Digital Audio Workstation (DAW)",
-    "license-type", GTK_LICENSE_GPL_3_0,
+    "license-type", GTK_LICENSE_AGPL_3_0,
+    "translator-credits", translators,
     "website", "https://www.zrythm.org",
-    "website-label", "Official Website",
+    "website-label", "Home Page",
     "version", "v" PACKAGE_VERSION,
     NULL);
   gtk_widget_destroy (GTK_WIDGET (img));
@@ -669,14 +702,15 @@ on_timeline_clipboard_received (
     return;
   TimelineSelections * ts =
     timeline_selections_deserialize (text);
+  timeline_selections_post_deserialize (ts);
   g_message ("printing deserialized");
   timeline_selections_print (ts);
 
   if (MAIN_WINDOW->last_focused ==
       GTK_WIDGET (MW_TIMELINE))
     {
-      timeline_selections_paste_to_pos (ts,
-                                        &PLAYHEAD);
+      timeline_selections_paste_to_pos (
+        ts, PLAYHEAD);
     }
 
   /* free ts */
@@ -710,7 +744,7 @@ activate_delete (GSimpleAction *action,
                             action);
     }
   else if (MAIN_WINDOW->last_focused ==
-             GTK_WIDGET (MIDI_ARRANGER))
+             GTK_WIDGET (MW_MIDI_ARRANGER))
     {
       UndoableAction * action =
         delete_midi_arranger_selections_action_new (
@@ -730,7 +764,7 @@ activate_duplicate (GSimpleAction *action,
     {
     }
   else if (MAIN_WINDOW->last_focused ==
-             GTK_WIDGET (MIDI_ARRANGER))
+             GTK_WIDGET (MW_MIDI_ARRANGER))
     {
     }
 }
@@ -750,7 +784,7 @@ activate_select_all (
 	gpointer user_data)
 {
 	if (MAIN_WINDOW->last_focused
-		== GTK_WIDGET (MIDI_ARRANGER))
+		== GTK_WIDGET (MW_MIDI_ARRANGER))
 	{
 		midi_arranger_widget_select_all (
 			Z_MIDI_ARRANGER_WIDGET (
@@ -1042,3 +1076,121 @@ change_state_loop (
 
   transport_set_loop (TRANSPORT, loop);
 }
+
+void
+change_state_metronome (
+  GSimpleAction * action,
+  GVariant *      value,
+  gpointer        user_data)
+{
+  int enabled = g_variant_get_boolean (value);
+
+  g_simple_action_set_state (action, value);
+
+  g_settings_set_int (
+    S_UI,
+    "metronome-enabled",
+    enabled);
+  transport_set_metronome_enabled (
+    TRANSPORT, enabled);
+}
+
+void
+activate_quick_quantize (
+  GSimpleAction *action,
+  GVariant      * _variant,
+  gpointer       user_data)
+{
+  g_return_if_fail (_variant);
+
+  gsize size;
+  const char * variant =
+    g_variant_get_string (_variant, &size);
+  if (string_is_equal (variant, "timeline", 1))
+    {
+      UndoableAction * ua =
+        (UndoableAction *)
+        quantize_timeline_selections_action_new (
+          TL_SELECTIONS,
+          QUANTIZE_OPTIONS_TIMELINE);
+      undo_manager_perform (
+        UNDO_MANAGER, ua);
+    }
+  else if (string_is_equal (variant, "editor", 1))
+    {
+    }
+  else if (string_is_equal (variant, "global", 1))
+    {
+      if (MAIN_WINDOW->last_focused ==
+          GTK_WIDGET (MW_TIMELINE))
+        {
+          UndoableAction * ua =
+            (UndoableAction *)
+            quantize_timeline_selections_action_new (
+              TL_SELECTIONS,
+              QUANTIZE_OPTIONS_TIMELINE);
+          undo_manager_perform (
+            UNDO_MANAGER, ua);
+        }
+    }
+  else
+    {
+      g_return_if_reached ();
+    }
+  g_message ("quantize");
+}
+
+void
+activate_quantize_options (
+  GSimpleAction *action,
+  GVariant      *_variant,
+  gpointer       user_data)
+{
+  g_return_if_fail (_variant);
+
+  gsize size;
+  const char * variant =
+    g_variant_get_string (_variant, &size);
+  if (string_is_equal (variant, "timeline", 1))
+    {
+      QuantizeDialogWidget * quant =
+        quantize_dialog_widget_new (
+          QUANTIZE_OPTIONS_TIMELINE);
+      gtk_window_set_transient_for (
+        GTK_WINDOW (quant),
+        GTK_WINDOW (MAIN_WINDOW));
+      gtk_dialog_run (GTK_DIALOG (quant));
+      gtk_widget_destroy (GTK_WIDGET (quant));
+    }
+  else if (string_is_equal (variant, "editor", 1))
+    {
+      QuantizeDialogWidget * quant =
+        quantize_dialog_widget_new (
+          QUANTIZE_OPTIONS_EDITOR);
+      gtk_window_set_transient_for (
+        GTK_WINDOW (quant),
+        GTK_WINDOW (MAIN_WINDOW));
+      gtk_dialog_run (GTK_DIALOG (quant));
+      gtk_widget_destroy (GTK_WIDGET (quant));
+    }
+  else if (string_is_equal (variant, "global", 1))
+    {
+      if (MAIN_WINDOW->last_focused ==
+          GTK_WIDGET (MW_TIMELINE))
+        {
+          QuantizeDialogWidget * quant =
+            quantize_dialog_widget_new (
+              QUANTIZE_OPTIONS_TIMELINE);
+          gtk_window_set_transient_for (
+            GTK_WINDOW (quant),
+            GTK_WINDOW (MAIN_WINDOW));
+          gtk_dialog_run (GTK_DIALOG (quant));
+          gtk_widget_destroy (GTK_WIDGET (quant));
+        }
+    }
+  else
+    {
+      g_return_if_reached ();
+    }
+  g_message ("quantize opts");
+}
