@@ -49,6 +49,7 @@
 #include "utils/flags.h"
 #include "utils/math.h"
 #include "utils/objects.h"
+#include "utils/stoat.h"
 
 #include <gtk/gtk.h>
 
@@ -127,38 +128,38 @@ disconnect_midi_ins (
 }
 
 /**
- * Disconnect stereo in ports from stereo out.
+ * Disconnect stereo in ports from the fader.
  *
  * Used when there is no plugin in the channel.
  */
 static inline void
-disconnect_stereo_in_from_stereo_out (
+disconnect_stereo_in_from_fader (
   Channel * ch)
 {
   port_disconnect (
     ch->stereo_in->l,
-    ch->stereo_out->l);
+    ch->fader.stereo_in->l);
   port_disconnect (
     ch->stereo_in->r,
-    ch->stereo_out->r);
+    ch->fader.stereo_in->r);
 }
 
 /**
  * Connects the Channel's stereo in ports to its
- * stereo out ports.
+ * fader in ports.
  *
  * Used when deleting the only plugin left.
  */
 static inline void
-connect_stereo_in_to_stereo_out (
+connect_stereo_in_to_fader (
   Channel * ch)
 {
   port_connect (
     ch->stereo_in->l,
-    ch->stereo_out->l);
+    ch->fader.stereo_in->l);
   port_connect (
     ch->stereo_in->r,
-    ch->stereo_out->r);
+    ch->fader.stereo_in->r);
 }
 
 /**
@@ -393,10 +394,10 @@ done2:
 
 /**
  * Connect Plugin's outputs to the Channel's
- * stereo out ports.
+ * fader in ports.
  */
 static inline void
-connect_pl_to_stereo_out (
+connect_pl_to_fader (
   Plugin * pl,
   Channel * ch)
 {
@@ -414,10 +415,10 @@ connect_pl_to_stereo_out (
             {
               port_connect (
                 out_port,
-                ch->stereo_out->l);
+                ch->fader.stereo_in->l);
               port_connect (
                 out_port,
-                ch->stereo_out->r);
+                ch->fader.stereo_in->r);
               break;
             }
         }
@@ -437,14 +438,14 @@ connect_pl_to_stereo_out (
             {
               port_connect (
                 out_port,
-                ch->stereo_out->l);
+                ch->fader.stereo_in->l);
               last_index++;
             }
           else if (last_index == 1)
             {
               port_connect (
                 out_port,
-                ch->stereo_out->r);
+                ch->fader.stereo_in->r);
               break;
             }
         }
@@ -597,10 +598,10 @@ done2:
 
 /**
  * Disconnects the Plugin's output ports from
- * the Channel's stereo out.
+ * the Channel's fader out.
  */
 static inline void
-disconnect_pl_from_stereo_out (
+disconnect_pl_from_fader (
   Plugin * pl,
   Channel * ch)
 {
@@ -613,13 +614,13 @@ disconnect_pl_from_stereo_out (
       if (out_port->identifier.type == TYPE_AUDIO)
         {
           if (ports_connected (
-                out_port, ch->stereo_out->l))
+                out_port, ch->fader.stereo_in->l))
             port_disconnect (
-              out_port, ch->stereo_out->l);
+              out_port, ch->fader.stereo_in->l);
           if (ports_connected (
-                out_port, ch->stereo_out->r))
+                out_port, ch->fader.stereo_in->r))
             port_disconnect (
-              out_port, ch->stereo_out->r);
+              out_port, ch->fader.stereo_in->r);
         }
     }
 }
@@ -637,7 +638,7 @@ connect_no_prev_no_next (
    * ----------------------------------------- */
   /* channel stereo in is connected to channel
    * stereo out. disconnect it */
-  disconnect_stereo_in_from_stereo_out (ch);
+  disconnect_stereo_in_from_fader (ch);
 
   /* -------------------------------------------
    * connect input ports
@@ -651,7 +652,7 @@ connect_no_prev_no_next (
    * ------------------------------------*/
 
   /* connect plugin to stereo out */
-  connect_pl_to_stereo_out (pl, ch);
+  connect_pl_to_fader (pl, ch);
 }
 
 /**
@@ -700,7 +701,7 @@ connect_prev_no_next (
    * ----------------------------------------- */
   /* prev plugin is connected to channel stereo out.
    * disconnect it */
-  disconnect_pl_from_stereo_out (prev_pl, ch);
+  disconnect_pl_from_fader (prev_pl, ch);
 
   /* -------------------------------------------
    * connect input ports
@@ -715,7 +716,7 @@ connect_prev_no_next (
    * ------------------------------------*/
 
   /* connect plugin output ports to stereo_out */
-  connect_pl_to_stereo_out (pl, ch);
+  connect_pl_to_fader (pl, ch);
 }
 
 /**
@@ -772,14 +773,14 @@ disconnect_no_prev_no_next (
    * ------------------------------------*/
 
   /* disconnect plugin from stereo out */
-  disconnect_pl_from_stereo_out (pl, ch);
+  disconnect_pl_from_fader (pl, ch);
 
   /* -----------------------------------------
    * connect ports
    * ----------------------------------------- */
   /* channel stereo in should be connected to
    * channel stereo out. connect it */
-  connect_stereo_in_to_stereo_out (ch);
+  connect_stereo_in_to_fader (ch);
 }
 
 /**
@@ -838,14 +839,14 @@ disconnect_prev_no_next (
 
   /* disconnect plugin output ports from stereo
    * out */
-  disconnect_pl_from_stereo_out (pl, ch);
+  disconnect_pl_from_fader (pl, ch);
 
   /* -----------------------------------------
    * connect ports
    * ----------------------------------------- */
   /* prev plugin should be connected to channel
    * stereo out. connect it */
-  connect_pl_to_stereo_out (prev_pl, ch);
+  connect_pl_to_fader (prev_pl, ch);
 }
 
 /**
@@ -889,16 +890,16 @@ disconnect_prev_next (
  * The MidiEvents are already dequeued at this
  * point.
  */
-__attribute__((annotate("realtime")))
+REALTIME
 void
 channel_handle_recording (Channel * self)
 {
   Region * region =
-    track_get_region_at_pos (self->track,
-                             &PLAYHEAD);
+    track_get_region_at_pos (
+      self->track, PLAYHEAD);
   /* get end position TODO snap */
   Position tmp;
-  position_set_to_pos (&tmp, &PLAYHEAD);
+  position_set_to_pos (&tmp, PLAYHEAD);
   position_add_frames (
     &tmp,
     AUDIO_ENGINE->nframes + 1);
@@ -918,7 +919,7 @@ channel_handle_recording (Channel * self)
           /* create region */
           mr =
             midi_region_new (
-              &PLAYHEAD, &tmp, 1);
+              PLAYHEAD, &tmp, 1);
           track_add_region (
             self->track, mr, 0, F_GEN_NAME,
             F_GEN_WIDGET);
@@ -941,11 +942,11 @@ channel_handle_recording (Channel * self)
                   case MIDI_EVENT_TYPE_NOTE_ON:
                     mn =
                       midi_note_new (
-                        mr, &PLAYHEAD, &tmp,
+                        mr, PLAYHEAD, &tmp,
                         ev->note_pitch,
                         ev->velocity, 1);
                     midi_region_add_midi_note (
-                      mr, mn, F_GEN_WIDGET);
+                      mr, mn);
 
                     /* add to unended notes */
                     array_append (
@@ -994,6 +995,10 @@ channel_prepare_process (Channel * channel)
       port_clear_buffer (channel->midi_in);
       port_clear_buffer (channel->piano_roll);
     }
+  port_clear_buffer (channel->fader.stereo_in->l);
+  port_clear_buffer (channel->fader.stereo_in->r);
+  port_clear_buffer (channel->fader.stereo_out->l);
+  port_clear_buffer (channel->fader.stereo_out->r);
   port_clear_buffer (channel->stereo_out->l);
   port_clear_buffer (channel->stereo_out->r);
   port_clear_buffer (channel->midi_in);
@@ -1129,8 +1134,8 @@ _create_channel (
       TYPE_EVENT,
       FLOW_INPUT,
       pll);
-  self->midi_in->identifier.owner_type =
-    PORT_OWNER_TYPE_TRACK;
+  /*self->midi_in->identifier.owner_type =*/
+    /*PORT_OWNER_TYPE_TRACK;*/
   self->midi_in->midi_events =
     midi_events_new (
       self->midi_in);
@@ -1339,13 +1344,23 @@ channel_connect (
 
   if (ch->type == CT_BUS ||
       ch->type == CT_AUDIO ||
-      ch->type == CT_MASTER)
+      ch->type == CT_MASTER ||
+      ch->type == CT_MIDI)
     {
-      /* connect stereo in to stereo out */
-      port_connect (ch->stereo_in->l,
-                    ch->stereo_out->l);
-      port_connect (ch->stereo_in->r,
-                    ch->stereo_out->r);
+      /* connect stereo in to stereo out through
+       * fader */
+      port_connect (
+        ch->stereo_in->l,
+        ch->fader.stereo_in->l);
+      port_connect (
+        ch->stereo_in->r,
+        ch->fader.stereo_in->r);
+      port_connect (
+        ch->fader.stereo_out->l,
+        ch->stereo_out->l);
+      port_connect (
+        ch->fader.stereo_out->r,
+        ch->stereo_out->r);
     }
 
   if (ch->type != CT_MASTER)
