@@ -210,6 +210,34 @@ port_find_from_identifier (
           break;
         }
       break;
+    case PORT_OWNER_TYPE_FADER:
+      tr = TRACKLIST->tracks[id->track_pos];
+      g_warn_if_fail (tr);
+      ch = tr->channel;
+      g_warn_if_fail (ch);
+      if (id->flow == FLOW_OUTPUT)
+        {
+          if (id->flags & PORT_FLAG_STEREO_L)
+            return ch->fader.stereo_out->l;
+          else if (id->flags &
+                     PORT_FLAG_STEREO_R)
+            return ch->fader.stereo_out->r;
+        }
+      else if (id->flow == FLOW_INPUT)
+        {
+          if (id->flags & PORT_FLAG_STEREO_L)
+            return ch->fader.stereo_in->r;
+          else if (id->flags &
+                     PORT_FLAG_STEREO_R)
+            return ch->fader.stereo_in->r;
+        }
+      break;
+    case PORT_OWNER_TYPE_SAMPLE_PROCESSOR:
+      if (id->flags & PORT_FLAG_STEREO_L)
+        return SAMPLE_PROCESSOR->stereo_out->l;
+      else if (id->flags &
+                 PORT_FLAG_STEREO_R)
+        return SAMPLE_PROCESSOR->stereo_out->r;
     }
 
   g_return_val_if_reached (NULL);
@@ -301,6 +329,8 @@ port_get_all (
   _ADD (AUDIO_ENGINE->stereo_out->r);
   _ADD (AUDIO_ENGINE->midi_in);
   _ADD (AUDIO_ENGINE->midi_editor_manual_press);
+  _ADD (SAMPLE_PROCESSOR->stereo_out->l);
+  _ADD (SAMPLE_PROCESSOR->stereo_out->r);
 
   Plugin * pl;
   Channel * ch;
@@ -321,6 +351,12 @@ port_get_all (
       _ADD (ch->piano_roll);
       _ADD (ch->midi_in);
 
+      /* add fader ports */
+      _ADD (ch->fader.stereo_in->l);
+      _ADD (ch->fader.stereo_in->r);
+      _ADD (ch->fader.stereo_out->l);
+      _ADD (ch->fader.stereo_out->r);
+
 #define ADD_PLUGIN_PORTS \
           if (!pl) \
             continue; \
@@ -386,6 +422,52 @@ port_set_owner_plugin (
     PORT_OWNER_TYPE_PLUGIN;
 }
 
+/**
+ * Sets the owner sample processor.
+ */
+void
+port_set_owner_sample_processor (
+  Port *   port,
+  SampleProcessor * sample_processor)
+{
+  port->sample_processor = sample_processor;
+  port->identifier.owner_type =
+    PORT_OWNER_TYPE_SAMPLE_PROCESSOR;
+}
+
+/**
+ * Sets the owner track & its ID.
+ */
+void
+port_set_owner_track (
+  Port *    port,
+  Track *   track)
+{
+  g_warn_if_fail (port && track);
+
+  port->track = track;
+  port->identifier.track_pos = track->pos;
+  port->identifier.owner_type =
+    PORT_OWNER_TYPE_TRACK;
+}
+
+/**
+ * Sets the owner fader & its ID.
+ */
+void
+port_set_owner_fader (
+  Port *    port,
+  Fader *   fader)
+{
+  g_warn_if_fail (port && fader);
+
+  port->track = fader->channel->track;
+  port->identifier.track_pos =
+    fader->channel->track->pos;
+  port->identifier.owner_type =
+    PORT_OWNER_TYPE_FADER;
+}
+
 /**
  */
 /*void*/
@@ -558,13 +640,20 @@ ports_remove (
 
 /**
  * Apply given fader value to port.
+ *
+ * @param start_frame The start frame offset from
+ *   0 in this cycle.
+ * @param nframes The number of frames to process.
  */
 void
-port_apply_fader (Port * port, float amp)
+port_apply_fader (
+  Port *    port,
+  float     amp,
+  const int start_frame,
+  const int nframes)
 {
-  for (uint32_t i = 0;
-       i < AUDIO_ENGINE->block_length;
-       i++)
+  for (int i = start_frame;
+       i < start_frame + nframes; i++)
     {
       if (port->buf[i] != 0.f)
         port->buf[i] *= amp;
@@ -575,17 +664,32 @@ port_apply_fader (Port * port, float amp)
 /**
  * First sets port buf to 0, then sums the given
  * port signal from its inputs.
+ *
+ * @param start_frame The start frame offset from
+ *   0 in this cycle.
+ * @param nframes The number of frames to process.
+ * @param noroll Clear the port buffer in this
+ *   range.
  */
 void
-port_sum_signal_from_inputs (Port * port)
+port_sum_signal_from_inputs (
+  Port *    port,
+  const int start_frame,
+  const int nframes,
+  const int noroll)
 {
   Port * src_port;
-  int block_length = AUDIO_ENGINE->block_length;
   int k, l;
 
+  g_warn_if_fail (
+    start_frame + nframes <= AUDIO_ENGINE->nframes);
+
   switch (port->identifier.type)
     {
     case TYPE_EVENT:
+      if (noroll)
+        break;
+
       for (k = 0; k < port->num_srcs; k++)
         {
           src_port = port->srcs[k];
@@ -594,7 +698,8 @@ port_sum_signal_from_inputs (Port * port)
               TYPE_EVENT);
           midi_events_append (
             src_port->midi_events,
-            port->midi_events, 0);
+            port->midi_events, start_frame,
+            nframes, 0);
         }
 
       /* send UI notification */
@@ -608,19 +713,31 @@ port_sum_signal_from_inputs (Port * port)
               /*AUDIO_ENGINE->trigger_midi_activity = 1;*/
             /*}*/
           if (port->identifier.owner_type ==
-                     PORT_OWNER_TYPE_TRACK)
+                PORT_OWNER_TYPE_TRACK)
             {
               port->track->trigger_midi_activity = 1;
             }
         }
       break;
     case TYPE_AUDIO:
+      if (noroll)
+        {
+          for (l = start_frame; l < nframes; l++)
+            {
+              port->buf[l] = 0;
+            }
+          break;
+        }
+
       for (k = 0; k < port->num_srcs; k++)
         {
           src_port = port->srcs[k];
 
           /* sum the signals */
-          for (l = 0; l < block_length; l++)
+          /*g_message ("port %s start frame %ld nframes %d",*/
+                     /*port->identifier.label,*/
+                     /*start_frame, nframes);*/
+          for (l = start_frame; l < nframes; l++)
             {
               port->buf[l] += src_port->buf[l];
             }
@@ -771,16 +888,21 @@ port_apply_pan_stereo (Port *       l,
 }
 
 /**
- * Applies the pan to the given L/R ports.
+ * Applies the pan to the given port.
+ *
+ * @param start_frame The start frame offset from
+ *   0 in this cycle.
+ * @param nframes The number of frames to process.
  */
 void
 port_apply_pan (
   Port *       port,
   float        pan,
   PanLaw       pan_law,
-  PanAlgorithm pan_algo)
+  PanAlgorithm pan_algo,
+  const int    start_frame,
+  const int    nframes)
 {
-  int block_length = AUDIO_ENGINE->block_length;
   int i;
   float calc_r = 0.f, calc_l = 0.f;
   int is_stereo_r =
@@ -802,7 +924,8 @@ port_apply_pan (
       break;
     }
 
-  for (i = 0; i < block_length; i++)
+  for (i = start_frame;
+       i < start_frame + nframes; i++)
     {
       if (port->buf[i] == 0.f)
         continue;
