@@ -18,6 +18,8 @@
  */
 
 #include "audio/audio_region.h"
+#include "audio/automation_region.h"
+#include "audio/chord_region.h"
 #include "audio/channel.h"
 #include "audio/midi_note.h"
 #include "audio/midi_region.h"
@@ -26,8 +28,10 @@
 #include "audio/track.h"
 #include "ext/audio_decoder/ad.h"
 #include "gui/widgets/audio_region.h"
+#include "gui/widgets/automation_region.h"
 #include "gui/widgets/bot_dock_edge.h"
 #include "gui/widgets/center_dock.h"
+#include "gui/widgets/chord_region.h"
 #include "gui/widgets/clip_editor.h"
 #include "gui/widgets/main_window.h"
 #include "gui/widgets/midi_arranger.h"
@@ -63,12 +67,10 @@ ARRANGER_OBJ_DEFINE_MOVABLE_W_LENGTH (
 void
 region_init (
   Region *         self,
-  const RegionType type,
   const Position * start_pos,
   const Position * end_pos,
   const int        is_main)
 {
-  g_message ("creating region");
   position_set_to_pos (&self->start_pos,
                        start_pos);
   position_set_to_pos (&self->end_pos,
@@ -82,20 +84,11 @@ region_init (
   position_set_to_pos (&self->loop_end_pos,
                        &self->true_end_pos);
   self->linked_region_name = NULL;
-  self->type = type;
 
   if (is_main)
     {
-      if (type == REGION_TYPE_MIDI)
-        {
-          ARRANGER_OBJECT_SET_AS_MAIN (
-            REGION, Region, region);
-        }
-      else if (type == REGION_TYPE_AUDIO)
-        {
-          /* TODO */
-
-        }
+      ARRANGER_OBJECT_SET_AS_MAIN (
+        REGION, Region, region);
     }
 }
 
@@ -127,6 +120,30 @@ region_set_lane (
     }
 }
 
+/**
+ * Sets the automation track.
+ */
+void
+region_set_automation_track (
+  Region * region,
+  AutomationTrack * at)
+{
+  g_return_if_fail (at);
+
+  Region * r;
+  for (int i = 0; i < 2; i++)
+    {
+      if (i == AOI_COUNTERPART_MAIN)
+        r = region_get_main_region (region);
+      else if (i == AOI_COUNTERPART_MAIN_TRANSIENT)
+        r = region_get_main_trans_region (region);
+
+      r->at = at;
+      r->at_index = at->index;
+      r->track_pos = at->track->pos;
+    }
+}
+
 /**
  * Inits freshly loaded region.
  */
@@ -137,31 +154,53 @@ region_init_loaded (Region * self)
     region_find_by_name (
       self->linked_region_name);
 
-  if (self->type == REGION_TYPE_AUDIO)
+  int i;
+  switch (self->type)
     {
-      /* reload audio */
-      struct adinfo nfo;
-      SRC_DATA src_data;
-      long out_buff_size;
+    case REGION_TYPE_AUDIO:
+      {
+        /* reload audio */
+        struct adinfo nfo;
+        SRC_DATA src_data;
+        long out_buff_size;
 
-      audio_decode (
-        &nfo, &src_data,
-        &self->buff, &out_buff_size,
-        self->filename);
+        audio_decode (
+          &nfo, &src_data,
+          &self->buff, &out_buff_size,
+          self->filename);
 
-      self->buff_size =
-        src_data.output_frames_gen;
-      self->channels = nfo.channels;
-    }
-  else if (self->type == REGION_TYPE_MIDI)
-    {
-      MidiNote * mn;
-      for (int i = 0; i < self->num_midi_notes; i++)
-        {
-          mn = self->midi_notes[i];
-          mn->region = self;
-          midi_note_init_loaded (mn);
+        self->buff_size =
+          src_data.output_frames_gen;
+        self->channels = nfo.channels;
+      }
+      break;
+    case REGION_TYPE_MIDI:
+      {
+        MidiNote * mn;
+        for (i = 0; i < self->num_midi_notes; i++)
+          {
+            mn = self->midi_notes[i];
+            mn->region = self;
+            midi_note_init_loaded (mn);
+          }
+      }
+      break;
+    case REGION_TYPE_CHORD:
+      {
+        ChordObject * chord;
+        for (i = 0; i < self->num_chord_objects;
+             i++)
+          {
+            chord = self->chord_objects[i];
+            chord_object_init_loaded (chord);
+          }
         }
+      break;
+    case REGION_TYPE_AUTOMATION:
+      {
+        /* TODO */
+      }
+      break;
     }
 
   ARRANGER_OBJECT_SET_AS_MAIN (
@@ -204,11 +243,14 @@ region_find_by_name (
 {
   int i, j, k;
   Track * track;
+  AutomationTracklist * atl;
+  AutomationTrack * at;
   TrackLane * lane;
   Region * r;
   for (i = 0; i < TRACKLIST->num_tracks; i++)
     {
       track = TRACKLIST->tracks[i];
+      atl = &track->automation_tracklist;
       g_warn_if_fail (track);
 
       for (k = 0; k < track->num_lanes; k++)
@@ -222,7 +264,31 @@ region_find_by_name (
                 return r;
             }
         }
+
+      if (track->type == TRACK_TYPE_CHORD)
+        {
+          for (j = 0; j < track->num_chord_regions;
+               j++)
+            {
+              r = track->chord_regions[j];
+              if (!g_strcmp0 (r->name, name))
+                return r;
+            }
+        }
+
+      for (j = 0; j < atl->num_ats; j++)
+        {
+          at = atl->ats[j];
+
+          for (k = 0; k < at->num_regions; k++)
+            {
+              r = at->regions[k];
+              if (!g_strcmp0 (r->name, name))
+                return r;
+            }
+        }
     }
+
   return NULL;
 }
 
@@ -271,19 +337,22 @@ region_gen_widget (
       else if (i == 3)
         r = region_get_lane_trans_region (r);
 
+#define GEN_W(type,sc) \
+  case REGION_TYPE_##type: \
+    r->widget = \
+      Z_REGION_WIDGET ( \
+        sc##_region_widget_new (r)); \
+    break
+
       switch (r->type)
         {
-        case REGION_TYPE_MIDI:
-          r->widget =
-            Z_REGION_WIDGET (
-              midi_region_widget_new (r));
-          break;
-        case REGION_TYPE_AUDIO:
-          r->widget =
-            Z_REGION_WIDGET (
-              audio_region_widget_new (r));
-          break;
+        GEN_W (MIDI, midi);
+        GEN_W (AUDIO, audio);
+        GEN_W (AUTOMATION, automation);
+        GEN_W (CHORD, chord);
         }
+
+#undef GEN_W
     }
 }
 
@@ -318,12 +387,25 @@ region_get_full_length_in_ticks (
     position_to_ticks (&region->start_pos);
 }
 
+/**
+ * Returns the full length as it appears on the
+ * timeline in frames.
+ */
+long
+region_get_full_length_in_frames (
+  const Region * region)
+{
+  return
+    region->end_pos.frames -
+    region->start_pos.frames;
+}
+
 /**
  * Returns the length of the loop in ticks.
  */
 long
 region_get_loop_length_in_ticks (
-  Region * region)
+  const Region * region)
 {
   return
     position_to_ticks (&region->loop_end_pos) -
@@ -335,7 +417,7 @@ region_get_loop_length_in_ticks (
  */
 long
 region_get_loop_length_in_frames (
-  Region * region)
+  const Region * region)
 {
   return
     position_to_frames (&region->loop_end_pos) -
@@ -474,6 +556,160 @@ region_set_clip_start_pos (
     }
 }
 
+/**
+ * Splits the given Region at the given Position,
+ * deletes the original Region and adds 2 new
+ * Regions in the same parent (Track or
+ * AutomationTrack).
+ *
+ * The given region must be the main region, as this
+ * will create 2 new main regions.
+ *
+ * @param region The Region to split. This Region
+ *   will be deleted.
+ * @param pos The Position to split at.
+ * @param pos_is_local If the position is local (1)
+ *   or global (0).
+ * @param r1 Address to hold the pointer to the
+ *   newly created region 1.
+ * @param r2 Address to hold the pointer to the
+ *   newly created region 2.
+ */
+void
+region_split (
+  Region *         region,
+  const Position * pos,
+  const int        pos_is_local,
+  Region **        r1,
+  Region **        r2)
+{
+  /* create the new regions */
+  *r1 =
+    region_clone (region, REGION_CLONE_COPY_MAIN);
+  *r2 =
+    region_clone (region, REGION_CLONE_COPY_MAIN);
+
+  /* get global/local positions (the local pos
+   * is after traversing the loops) */
+  Position globalp, localp;
+  if (pos_is_local)
+    {
+      position_set_to_pos (&globalp, pos);
+      position_add_ticks (
+        &globalp, region->start_pos.total_ticks);
+      position_set_to_pos (&localp, pos);
+    }
+  else
+    {
+      position_set_to_pos (&globalp, pos);
+      long localp_frames =
+        region_timeline_frames_to_local (
+          region, globalp.frames, 1);
+      position_from_frames (
+        &localp, localp_frames);
+    }
+
+  /* for first region just set the end pos */
+  region_set_end_pos (
+    *r1, &globalp, AO_UPDATE_ALL);
+
+  region_set_clip_start_pos (
+    *r2, &localp, AO_UPDATE_ALL);
+  region_set_start_pos (
+    *r2, &globalp, AO_UPDATE_ALL);
+
+  /* add them to the track */
+  track_add_region (
+    region_get_track (region),
+    *r1, region->at, region->lane_pos, 1);
+  track_add_region (
+    region_get_track (region),
+    *r2, region->at, region->lane_pos, 1);
+
+  /* generate widgets so update visibility in the
+   * arranger can work */
+  region_gen_widget (*r1);
+  region_gen_widget (*r2);
+
+  /* select them */
+  timeline_selections_clear (TL_SELECTIONS);
+  timeline_selections_add_region (
+    TL_SELECTIONS, *r1);
+  timeline_selections_add_region (
+    TL_SELECTIONS, *r2);
+
+  /* change to r1 if the original region was the
+   * clip editor region */
+  if (CLIP_EDITOR->region == region)
+    {
+      clip_editor_set_region (
+        CLIP_EDITOR, *r1);
+    }
+
+  /* remove and free the original region */
+  track_remove_region (
+    region_get_track (region),
+    region, F_FREE);
+
+  EVENTS_PUSH (ET_REGION_CREATED, *r1);
+  EVENTS_PUSH (ET_REGION_CREATED, *r2);
+}
+
+/**
+ * Undoes what region_split() did.
+ *
+ * @param r1 The first region.
+ * @param r2 The second Region.
+ * @param region An address to store the newly
+ *   created Region.
+ */
+void
+region_unsplit (
+  Region *         r1,
+  Region *         r2,
+  Region **        region)
+{
+  /* create the new region */
+  *region =
+    region_clone (r1, REGION_CLONE_COPY_MAIN);
+
+  /* set the end pos to the end pos of r2 */
+  region_set_end_pos (
+    *region, &r2->end_pos, AO_UPDATE_ALL);
+
+  /* add it to the track */
+  track_add_region (
+    region_get_track (r1),
+    *region, r1->at, r1->lane_pos, 1);
+
+  /* generate widgets so update visibility in the
+   * arranger can work */
+  region_gen_widget (*region);
+
+  /* select it */
+  timeline_selections_clear (TL_SELECTIONS);
+  timeline_selections_add_region (
+    TL_SELECTIONS, *region);
+
+  /* change to r1 if the original region was the
+   * clip editor region */
+  if (CLIP_EDITOR->region == r1)
+    {
+      clip_editor_set_region (
+        CLIP_EDITOR, *region);
+    }
+
+  /* remove and free the original regions */
+  track_remove_region (
+    region_get_track (r1),
+    r1, F_FREE);
+  track_remove_region (
+    region_get_track (r2),
+    r2, F_FREE);
+
+  EVENTS_PUSH (ET_REGION_CREATED, *region);
+}
+
 /**
  * Sets Region name (without appending anything to
  * it) to all associated regions.
@@ -555,17 +791,42 @@ region_at_position (
 /**
  * Returns if the position is inside the region
  * or not.
+ *
+ * @param gframes Global position in frames.
  */
 int
 region_is_hit (
-  Region *   region,
-  Position * pos) ///< global position
+  const Region * region,
+  const long     gframes)
 {
   return
-    position_compare (
-      &region->start_pos, pos) <= 0 &&
-    position_compare (
-      &region->end_pos, pos) > 0;
+    region->start_pos.frames <= gframes &&
+    region->end_pos.frames > gframes;
+}
+
+/**
+ * Returns if any part of the Region is inside the
+ * given range, inclusive.
+ */
+int
+region_is_hit_by_range (
+  const Region * region,
+  const long     gframes_start,
+  const long     gframes_end)
+{
+  /* 4 cases:
+   * - region start is inside range
+   * - region end is inside range
+   * - start is inside region
+   * - end is inside region
+   */
+  return
+    (gframes_start <= region->start_pos.frames &&
+     gframes_end > region->start_pos.frames) ||
+    (gframes_start <= region->end_pos.frames &&
+     gframes_end > region->end_pos.frames) ||
+    region_is_hit (region, gframes_start) ||
+    region_is_hit (region, gframes_end);
 }
 
 /**
@@ -638,46 +899,119 @@ region_clone (
   Region *        region,
   RegionCloneFlag flag)
 {
-  int is_main = 0;
+  int is_main = 0, i, j;
   if (flag == REGION_CLONE_COPY_MAIN)
     is_main = 1;
 
   Region * new_region = NULL;
-  if (region->type == REGION_TYPE_MIDI)
+  switch (region->type)
     {
-      MidiRegion * mr =
-        midi_region_new (
-          &region->start_pos,
-          &region->end_pos,
-          is_main);
-      MidiRegion * mr_orig = region;
-      if (flag == REGION_CLONE_COPY ||
-          flag == REGION_CLONE_COPY_MAIN)
-        {
-          for (int i = 0;
-               i < mr_orig->num_midi_notes; i++)
-            {
-              MidiNote * mn =
-                midi_note_clone (
-                  mr_orig->midi_notes[i],
-                  MIDI_NOTE_CLONE_COPY_MAIN);
+    case REGION_TYPE_MIDI:
+      {
+        MidiRegion * mr =
+          midi_region_new (
+            &region->start_pos,
+            &region->end_pos,
+            is_main);
+        MidiRegion * mr_orig = region;
+        if (flag == REGION_CLONE_COPY ||
+            flag == REGION_CLONE_COPY_MAIN)
+          {
+            for (i = 0;
+                 i < mr_orig->num_midi_notes; i++)
+              {
+                MidiNote * mn =
+                  midi_note_clone (
+                    mr_orig->midi_notes[i],
+                    MIDI_NOTE_CLONE_COPY_MAIN);
 
-              midi_region_add_midi_note (
-                mr, mn, F_NO_GEN_WIDGET);
-            }
-        }
+                midi_region_add_midi_note (
+                  mr, mn);
+              }
+          }
 
-      new_region = (Region *) mr;
-    }
-  else if (region->type == REGION_TYPE_AUDIO)
-    {
-      Region * ar =
-        audio_region_new (
-          region->filename,
-          &region->start_pos,
-          is_main);
+        new_region = (Region *) mr;
+      }
+    break;
+    case REGION_TYPE_AUDIO:
+      {
+        Region * ar =
+          audio_region_new (
+            region->filename,
+            &region->start_pos,
+            is_main);
 
-      new_region = ar;
+        new_region = ar;
+      }
+    break;
+    case REGION_TYPE_AUTOMATION:
+      {
+        Region * ar  =
+          automation_region_new (
+            &region->start_pos,
+            &region->end_pos,
+            is_main);
+        Region * ar_orig = region;
+
+        AutomationPoint * src_ap, * dest_ap;
+        AutomationCurve * src_ac, * dest_ac;
+
+        /* add automation points */
+        for (j = 0; j < ar_orig->num_aps; j++)
+          {
+            src_ap = ar_orig->aps[j];
+            dest_ap =
+              automation_point_new_float (
+                src_ap->fvalue,
+                &src_ap->pos, F_MAIN);
+            automation_region_add_ap (
+              ar, dest_ap, 0);
+          }
+
+        /* add automation curves */
+        for (j = 0; j < ar_orig->num_acs; j++)
+          {
+            src_ac = ar_orig->acs[j];
+            dest_ac =
+              automation_curve_new (
+                ar_orig->at->automatable->type,
+                &src_ac->pos);
+            automation_region_add_ac (
+              ar, dest_ac);
+          }
+
+        new_region = ar;
+      }
+      break;
+    case REGION_TYPE_CHORD:
+      {
+        Region * cr =
+          chord_region_new (
+            &region->start_pos,
+            &region->end_pos,
+            is_main);
+        Region * cr_orig = region;
+        if (flag == REGION_CLONE_COPY ||
+            flag == REGION_CLONE_COPY_MAIN)
+          {
+            ChordObject * co;
+            for (i = 0;
+                 i < cr_orig->num_chord_objects;
+                 i++)
+              {
+                co =
+                  chord_object_clone (
+                    cr_orig->chord_objects[i],
+                    CHORD_OBJECT_CLONE_COPY_MAIN);
+
+                chord_region_add_chord_object (
+                  cr, co);
+              }
+          }
+
+        new_region = cr;
+      }
+      break;
     }
 
   /* set caches */
@@ -715,78 +1049,76 @@ region_clone (
     {
       new_region->track_pos = region->track_pos;
     }
+  new_region->at_index = region->at_index;
 
   return new_region;
 }
 
 /**
- * Converts a position on the timeline (global)
- * to a local position (in the clip).
+ * Converts frames on the timeline (global)
+ * to local frames (in the clip).
  *
  * If normalize is 1 it will only return a position
- * from 0.0.0.0 to loop_end (it will traverse the
+ * from 0 to loop_end (it will traverse the
  * loops to find the appropriate position),
  * otherwise it may exceed loop_end.
  *
- * @param timeline_pos Timeline position.
- * @param local_pos Position to fill.
+ * @param timeline_frames Timeline position in
+ *   frames.
+ *
+ * @return The local frames.
  */
-void
-region_timeline_pos_to_local (
-  Region *   region,
-  Position * timeline_pos,
-  Position * local_pos,
-  int        normalize)
+long
+region_timeline_frames_to_local (
+  const Region * region,
+  const long     timeline_frames,
+  const int      normalize)
 {
-  long diff_ticks;
+  long diff_frames;
 
   if (normalize)
     {
       if (region)
         {
-          diff_ticks =
-            position_to_ticks (timeline_pos) -
-            position_to_ticks (
-              &region->start_pos);
-          long loop_end_ticks =
-            position_to_ticks (
-              &region->loop_end_pos);
-          long clip_start_ticks =
-            position_to_ticks (
-              &region->clip_start_pos);
+          diff_frames =
+            timeline_frames -
+            region->start_pos.frames;
+          long loop_end_frames =
+            region->loop_end_pos.frames;
+          long clip_start_frames =
+            region->clip_start_pos.frames;
           long loop_size =
-            region_get_loop_length_in_ticks (
+            region_get_loop_length_in_frames (
               region);
 
-          diff_ticks += clip_start_ticks;
+          diff_frames += clip_start_frames;
 
-          while (diff_ticks >= loop_end_ticks)
+          while (diff_frames >= loop_end_frames)
             {
-              diff_ticks -= loop_size;
+              diff_frames -= loop_size;
             }
         }
       else
         {
-          diff_ticks = 0;
+          diff_frames = 0;
         }
-      position_from_ticks (local_pos, diff_ticks);
-      return;
+
+      return diff_frames;
     }
   else
     {
       if (region)
         {
-          diff_ticks =
-            position_to_ticks (timeline_pos) -
-            position_to_ticks (
-              &region->start_pos);
+          diff_frames =
+            timeline_frames -
+            region->start_pos.frames;
         }
       else
         {
-          diff_ticks = 0;
+          diff_frames = 0;
         }
-      position_from_ticks (local_pos, diff_ticks);
-      return;
+
+      return diff_frames;
     }
 }
 
@@ -847,10 +1179,8 @@ region_disconnect (
     }
   if (TL_SELECTIONS)
     {
-      array_delete (
-        TL_SELECTIONS->regions,
-        TL_SELECTIONS->num_regions,
-        self);
+      timeline_selections_remove_region (
+        TL_SELECTIONS, self);
     }
   if (MW_TIMELINE->start_region == self)
     MW_TIMELINE->start_region = NULL;
@@ -879,10 +1209,21 @@ region_free (Region * self)
       self->widget = NULL;
       region_widget_delete (widget);
     }
-  if (self->type == REGION_TYPE_MIDI)
-    midi_region_free_members (self);
-  if (self->type == REGION_TYPE_AUDIO)
-    audio_region_free_members (self);
+
+#define FREE_R(type,sc) \
+  case REGION_TYPE_##type: \
+    sc##_region_free_members (self); \
+  break
+
+  switch (self->type)
+    {
+      FREE_R (MIDI, midi);
+      FREE_R (AUDIO, audio);
+      FREE_R (CHORD, chord);
+      FREE_R (AUTOMATION, automation);
+    }
+
+#undef FREE_R
 
   free (self);
 }
