@@ -39,6 +39,7 @@
 #include "audio/track.h"
 #include "project.h"
 #include "utils/arrays.h"
+#include "utils/stoat.h"
 
 #ifdef HAVE_JACK
 #include <jack/thread.h>
@@ -111,6 +112,15 @@ node_finish (
 	for (int i = 0; i < self->n_childnodes; ++i)
     {
       /*g_message ("notifying dest %d", i);*/
+
+      /* set the largest playback latency of this
+       * route to the child as well */
+      self->childnodes[i]->route_playback_latency =
+        self->playback_latency;
+        /*MAX (*/
+          /*self->playback_latency,*/
+          /*self->childnodes[i]->*/
+            /*route_playback_latency);*/
       node_trigger (self->childnodes[i]);
       feeds = 1;
     }
@@ -124,8 +134,42 @@ node_finish (
     }
 }
 
+/**
+ * Returns a human friendly name of the node.
+ *
+ * Must be free'd.
+ */
+static char *
+get_node_name (
+  GraphNode * node)
+{
+  switch (node->type)
+    {
+    case ROUTE_NODE_TYPE_PLUGIN:
+      return
+        g_strdup (node->pl->descr->name);
+      break;
+    case ROUTE_NODE_TYPE_PORT:
+      return
+        g_strdup (node->port->identifier.label);
+      break;
+    case ROUTE_NODE_TYPE_FADER:
+      return
+        g_strdup_printf (
+          "%s Fader",
+          node->fader->channel->track->name);
+      break;
+    case ROUTE_NODE_TYPE_SAMPLE_PROCESSOR:
+      return
+        g_strdup ("Sample Processor");
+      break;
+    }
+  g_return_val_if_reached (NULL);
+}
+
 static void
-print_node (GraphNode * node)
+print_node (
+  GraphNode * node)
 {
   GraphNode * dest;
   if (!node)
@@ -134,29 +178,27 @@ print_node (GraphNode * node)
       return;
     }
 
+  char * name = get_node_name (node);
   char * str1 =
     g_strdup_printf (
-      "node [(%d) %s] refcount: %d | terminal: %s | initial: %s",
+      "node [(%d) %s] refcount: %d | terminal: %s | initial: %s | playback latency: %ld",
       node->id,
-      node->type == ROUTE_NODE_TYPE_PLUGIN ?
-        node->pl->descr->name :
-        node->port->identifier.label,
+      name,
       node->refcount,
       node->terminal ? "yes" : "no",
-      node->initial ? "yes" : "no");
+      node->initial ? "yes" : "no",
+      node->playback_latency);
+  g_free (name);
   char * str2;
   for (int j = 0; j < node->n_childnodes; j++)
     {
       dest = node->childnodes[j];
+      name = get_node_name (dest);
       str2 =
         g_strdup_printf ("%s (dest [(%d) %s])",
-          str1,
-          dest->id,
-          dest->type ==
-            ROUTE_NODE_TYPE_PLUGIN ?
-              dest->pl->descr->name :
-              dest->port->identifier.label);
+          str1, dest->id, name);
       g_free (str1);
+      g_free (name);
       str1 = str2;
     }
   g_message ("%s", str1);
@@ -169,27 +211,94 @@ print_node (GraphNode * node)
  */
 static void
 node_process (
-  GraphNode * node)
+  GraphNode * node,
+  const int   nframes)
 {
-  int i;
+  int i, j, noroll = 0;
   Channel * chan;
 
-  /*g_message ("num trigger nodes %d, max_trigger nodes %d", node->graph->n_trigger_queue, node->graph->trigger_queue_size);*/
-  /*for (i = 0; i < node->graph->n_trigger_queue; i++)*/
-    /*{*/
-      /*GraphNode * n = node->graph->trigger_queue[i];*/
-      /*g_message ("trigger node %d: %s",*/
-                 /*i,*/
-                 /*n->type == ROUTE_NODE_TYPE_PORT ?*/
-                 /*n->port->label :*/
-                 /*n->pl->descr->name);*/
-    /*}*/
+  int local_offset =
+    node->graph->router->local_offset;
+
+  /* figure out if we are doing a no-roll */
+  if (node->route_playback_latency <
+      AUDIO_ENGINE->remaining_latency_preroll)
+    {
+      /* no roll */
+      if (node->type == ROUTE_NODE_TYPE_PLUGIN)
+        {
+      /*g_message (*/
+        /*"-- not processing: %s "*/
+        /*"route latency %ld",*/
+        /*node->type == ROUTE_NODE_TYPE_PLUGIN ?*/
+          /*node->pl->descr->name :*/
+          /*node->port->identifier.label,*/
+        /*node->route_playback_latency);*/
+        }
+      noroll = 1;
+
+      /* if no-roll, only process terminal nodes
+       * to set their buffers to 0 */
+      if (!node->terminal)
+        return;
+    }
+  /*g_message ("processing");*/
+  /*print_node (node);*/
+
+  /* global positions in frames (samples) */
+  long g_start_frames;
+
+  /* only compensate latency when rolling */
+  if (TRANSPORT->play_state ==
+        PLAYSTATE_ROLLING)
+    {
+      /* if the playhead is before the loop-end
+       * point and the latency-compensated position
+       * is after the loop-end point it means that
+       * the loop was crossed, so compensate for
+       * that.
+       *
+       * if the position is before loop-end and
+       * position + frames is after loop end (there
+       * is a loop inside the range), that should be
+       * handled by the ports/processors instead */
+      g_start_frames =
+        transport_frames_add_frames (
+          TRANSPORT, PLAYHEAD->frames,
+          node->route_playback_latency -
+          AUDIO_ENGINE->remaining_latency_preroll);
+    }
+  else
+    {
+      g_start_frames = PLAYHEAD->frames;
+    }
 
   if (node->type == ROUTE_NODE_TYPE_PLUGIN)
     {
       /*g_message ("processing plugin %s",*/
                  /*node->pl->descr->name);*/
-      plugin_process (node->pl);
+      plugin_process (
+        node->pl, g_start_frames, nframes);
+      /*g_message (*/
+        /*"processing: %s at %ld "*/
+        /*"(route latency %ld)",*/
+        /*node->type == ROUTE_NODE_TYPE_PLUGIN ?*/
+          /*node->pl->descr->name :*/
+          /*node->port->identifier.label,*/
+        /*g_start_frames,*/
+        /*node->route_playback_latency);*/
+    }
+  else if (node->type == ROUTE_NODE_TYPE_FADER)
+    {
+      fader_process (
+        node->fader, local_offset, nframes);
+    }
+  else if (node->type ==
+           ROUTE_NODE_TYPE_SAMPLE_PROCESSOR)
+    {
+      sample_processor_process (
+        node->sample_processor, local_offset,
+        nframes);
     }
   else if (node->type == ROUTE_NODE_TYPE_PORT)
     {
@@ -226,10 +335,12 @@ node_process (
 
                   /* fill midi events to pass to
                    * ins plugin */
+                  /*g_message ("filling PR events");*/
                   instrument_track_fill_midi_events (
                     (InstrumentTrack *)chan->track,
-                    &PLAYHEAD,
-                    AUDIO_ENGINE->block_length,
+                    g_start_frames,
+                    local_offset,
+                    nframes,
                     port->midi_events);
                 }
               midi_events_dequeue (
@@ -286,13 +397,17 @@ node_process (
             case TRACK_TYPE_MARKER:
             case TRACK_TYPE_MIDI:
             default:
-              port_sum_signal_from_inputs (port);
+              port_sum_signal_from_inputs (
+                port, local_offset,
+                nframes, noroll);
               break;
             }
         }
 
       /* if channel stereo out port */
       else if (port->track &&
+          port->identifier.owner_type ==
+            PORT_OWNER_TYPE_TRACK &&
           port->identifier.type == TYPE_AUDIO &&
           port->identifier.flow == FLOW_OUTPUT)
         {
@@ -302,29 +417,16 @@ node_process (
                    !port->track->solo &&
                    port->track != P_MASTER_TRACK))
             {
-              /* (already cleared) */
+              /* TODO */
               /*port_clear_buffer (port);*/
             }
           /* if not muted/soloed process it */
           else
             {
               port_sum_signal_from_inputs (
-                port);
-
-              /* apply pan */
-              port_apply_pan (
-                port,
-                port->track->channel->fader.pan,
-                AUDIO_ENGINE->pan_law,
-                AUDIO_ENGINE->pan_algo);
-
-              /* apply fader */
-              port_apply_fader (
-                port,
-                port->track->channel->fader.amp);
+                port, local_offset,
+                nframes, noroll);
             }
-          /*g_atomic_int_set (*/
-            /*&port->owner_ch->processed, 1);*/
         }
 
       /* if JACK stereo out */
@@ -347,15 +449,22 @@ node_process (
                       JACK_PORT_T (port->data),
                       AUDIO_ENGINE->nframes);
 
-                  /* by this time, the Master channel should have its
-                   * Stereo Out ports filled. pass their buffers to JACK's
+                  /* by this time, the Master
+                   * channel should have its
+                   * Stereo Out ports filled.
+                   * pass their buffers to JACK's
                    * buffers */
-                  int nframes = AUDIO_ENGINE->nframes;
-                  for (i = 0;
-                       i < nframes;
+                  for (i = 0; i < port->num_srcs;
                        i++)
                     {
-                      out[i] = port->srcs[0]->buf[i];
+                      for (j = local_offset;
+                           j < local_offset +
+                             nframes;
+                           j++)
+                        {
+                          out[j] +=
+                            port->srcs[i]->buf[j];
+                        }
                     }
 
                   /* avoid unused warnings */
@@ -368,18 +477,18 @@ node_process (
                   /* write interleaved */
                   if (port ==
                         AUDIO_ENGINE->stereo_out->l)
-                    for (i = 0;
-                         i < AUDIO_ENGINE->nframes;
-                         i++)
+                  for (i = local_offset;
+                       i < local_offset + nframes;
+                       i++)
                       {
                         AUDIO_ENGINE->alsa_out_buf[i * 2] =
                           port->srcs[0]->buf[i];
                       }
                   else if (port ==
                         AUDIO_ENGINE->stereo_out->r)
-                    for (i = 0;
-                         i < AUDIO_ENGINE->nframes;
-                         i++)
+                  for (i = local_offset;
+                       i < local_offset + nframes;
+                       i++)
                       {
                         AUDIO_ENGINE->alsa_out_buf[i * 2 + 1] =
                           port->srcs[0]->buf[i];
@@ -412,7 +521,9 @@ node_process (
 
       else
         {
-          port_sum_signal_from_inputs (port);
+          port_sum_signal_from_inputs (
+            port, local_offset,
+            nframes, noroll);
         }
     }
 }
@@ -421,7 +532,8 @@ static inline void
 node_run (GraphNode * self)
 {
   /*graph_print (self->graph);*/
-  node_process (self);
+  node_process (
+    self, self->graph->router->nsamples);
   node_finish (self);
 }
 
@@ -444,10 +556,20 @@ add_feeds (
 
 static inline void
 add_depends (
-  GraphNode * self)
+  GraphNode * self,
+  GraphNode * src)
 {
   ++self->init_refcount;
   self->refcount = self->init_refcount;
+
+  /* add parent nodes */
+  self->parentnodes =
+    (GraphNode **) realloc (
+      self->parentnodes,
+      (self->init_refcount) *
+        sizeof (GraphNode *));
+  self->parentnodes[self->init_refcount - 1] =
+    src;
 }
 
 /*static int cnt = 0;*/
@@ -535,7 +657,7 @@ graph_worker_thread (void * g)
 	return 0;
 }
 
-__attribute__((annotate("realtime")))
+REALTIME
 static void *
 graph_main_thread (void * arg)
 {
@@ -669,6 +791,41 @@ graph_init_threads (
   sched_yield ();
 }
 
+/**
+ * Returns the max playback latency of the trigger
+ * nodes.
+ */
+static long
+graph_get_max_playback_latency (
+  Graph * graph)
+{
+  long max = 0;
+  GraphNode * node;
+  for (int i = 0; i < graph->n_init_triggers; i++)
+    {
+      node =  graph->init_trigger_list[i];
+      if (node->playback_latency > max)
+        max = node->playback_latency;
+    }
+
+  return max;
+}
+
+/**
+ * Returns the max playback latency of the trigger
+ * nodes.
+ */
+long
+router_get_max_playback_latency (
+  Router * router)
+{
+  router->max_playback_latency =
+    graph_get_max_playback_latency (
+      router->graph2);
+
+  return router->max_playback_latency;
+}
+
 static void
 node_connect (
   GraphNode * from,
@@ -680,7 +837,7 @@ node_connect (
     return;
 
   add_feeds (from, to);
-  add_depends (to);
+  add_depends (to, from);
 }
 
 static GraphNode *
@@ -715,6 +872,40 @@ find_node_from_plugin (
   return NULL;
 }
 
+static GraphNode *
+find_node_from_fader (
+  Graph * graph,
+  Fader * fader)
+{
+  GraphNode * node;
+  for (int i = 0; i < graph->n_graph_nodes; i++)
+    {
+      node = graph->graph_nodes[i];
+      if (node->type == ROUTE_NODE_TYPE_FADER &&
+          node->fader == fader)
+        return node;
+    }
+  return NULL;
+}
+
+static GraphNode *
+find_node_from_sample_processor (
+  Graph * graph,
+  SampleProcessor * sample_processor)
+{
+  GraphNode * node;
+  for (int i = 0; i < graph->n_graph_nodes; i++)
+    {
+      node = graph->graph_nodes[i];
+      if (node->type ==
+            ROUTE_NODE_TYPE_SAMPLE_PROCESSOR &&
+          node->sample_processor ==
+            sample_processor)
+        return node;
+    }
+  return NULL;
+}
+
 static inline GraphNode *
 graph_node_new (
   Graph * graph,
@@ -730,6 +921,12 @@ graph_node_new (
     node->pl = (Plugin *) data;
   else if (type == ROUTE_NODE_TYPE_PORT)
     node->port = (Port *) data;
+  else if (type == ROUTE_NODE_TYPE_FADER)
+    node->fader = (Fader *) data;
+  else if (type ==
+             ROUTE_NODE_TYPE_SAMPLE_PROCESSOR)
+    node->sample_processor =
+      (SampleProcessor *) data;
 
   return node;
 }
@@ -767,6 +964,8 @@ graph_add_terminal_node (
   GraphNode * node =
     graph_add_node (self, type, data);
   node->terminal = 1;
+  self->terminal_nodes[
+    self->terminal_node_count - 1] = node;
   /*g_message ("adding terminal node %d",*/
              /*node->id);*/
   return node;
@@ -942,10 +1141,17 @@ graph_reached_terminal_node (
 
 /**
  * Starts a new cycle.
+ *
+ * @param local_offset The local offset to start
+ *   playing from in this cycle:
+ *   (0 - <engine buffer size>)
  */
 void
 router_start_cycle (
-  Router * self)
+  Router *         self,
+  const int        nsamples,
+  const int        local_offset,
+  const Position * pos)
 {
   /*self->n_trigger_queue = 0;*/
   /*g_message ("num trigger nodes at start: %d, num init trigger nodes at start: %d, num_max trigger nodes at start: %d",*/
@@ -957,6 +1163,11 @@ router_start_cycle (
     return;
 
   self->graph1 = self->graph2;
+  self->nsamples = nsamples;
+  self->global_offset =
+    self->max_playback_latency -
+    AUDIO_ENGINE->remaining_latency_preroll;
+  self->local_offset = local_offset;
   zix_sem_post (&self->graph1->callback_start);
   /*g_message ("waiting callback done");*/
   /*for (int i = 0; i < self->num_threads; i++)*/
@@ -988,10 +1199,12 @@ add_port (
   Graph * self,
   Port *   port)
 {
+  PortOwnerType owner = port->identifier.owner_type;
+
   /*add_port_node (self, port);*/
   if (port->num_dests == 0 &&
       port->num_srcs == 0 &&
-      port->plugin)
+      owner == PORT_OWNER_TYPE_PLUGIN)
     {
       if (port->identifier.flow == FLOW_INPUT)
         graph_add_initial_node (
@@ -1002,17 +1215,30 @@ add_port (
     }
   else if (port->num_dests == 0 &&
            port->num_srcs == 0 &&
-           !port->plugin)
+           owner != PORT_OWNER_TYPE_PLUGIN &&
+           owner != PORT_OWNER_TYPE_FADER)
     {
     }
   else if (port->num_srcs == 0 &&
-      !(port->plugin &&
+      !(owner == PORT_OWNER_TYPE_PLUGIN &&
+        port->identifier.flow == FLOW_OUTPUT) &&
+      !(owner == PORT_OWNER_TYPE_FADER &&
+        port->identifier.flow == FLOW_OUTPUT) &&
+      !(owner ==
+          PORT_OWNER_TYPE_SAMPLE_PROCESSOR &&
         port->identifier.flow == FLOW_OUTPUT))
     graph_add_initial_node (
       self, ROUTE_NODE_TYPE_PORT, port);
   else if (port->num_dests == 0 &&
            port->num_srcs > 0 &&
-           !(port->plugin &&
+           !(owner == PORT_OWNER_TYPE_PLUGIN &&
+             port->identifier.flow ==
+               FLOW_INPUT) &&
+           !(owner == PORT_OWNER_TYPE_FADER &&
+             port->identifier.flow ==
+               FLOW_INPUT) &&
+           !(owner ==
+               PORT_OWNER_TYPE_SAMPLE_PROCESSOR &&
              port->identifier.flow == FLOW_INPUT))
     graph_add_terminal_node (
       self, ROUTE_NODE_TYPE_PORT, port);
@@ -1051,6 +1277,61 @@ connect_port (
     }
 }
 
+/**
+ * Returns the latency of only the given port,
+ * without adding the previous/next latencies.
+ *
+ * It returns the plugin's latency if plugin,
+ * otherwise 0.
+ */
+long
+get_node_single_playback_latency (
+  GraphNode * node)
+{
+  if (node->type == ROUTE_NODE_TYPE_PLUGIN)
+    {
+      /* latency is already set at this point */
+      return node->pl->latency;
+    }
+
+  return 0;
+}
+
+/**
+ * Sets the playback latency of the given node
+ * recursively.
+ *
+ * Used only when (re)creating the graph.
+ *
+ * @param dest_latency The total destination
+ * latency so far.
+ */
+void
+set_node_playback_latency (
+  GraphNode * node,
+  long        dest_latency)
+{
+  long max_latency = 0, parent_latency;
+
+  /* set this node's latency */
+  node->playback_latency =
+    dest_latency +
+    get_node_single_playback_latency (node);
+
+  /* set route playback latency if trigger node */
+  if (node->initial)
+    node->route_playback_latency =
+      node->playback_latency;
+
+  GraphNode * parent;
+  for (int i = 0; i < node->init_refcount; i++)
+    {
+      parent = node->parentnodes[i];
+      set_node_playback_latency (
+        parent, node->playback_latency);
+    }
+}
+
 Graph *
 graph_new (
   Router * router)
@@ -1061,19 +1342,15 @@ graph_new (
   g_warn_if_fail (router);
   self->router = router;
 
-  self->port_nodes =
-    g_hash_table_new (
-      g_int_hash,
-      g_int_equal);
-  self->plugin_nodes =
-    g_hash_table_new (
-      g_int_hash,
-      g_int_equal);
-
   /* ========================
    * first add all the nodes
    * ======================== */
 
+  /* add the sample processor */
+  graph_add_initial_node ( \
+    self, ROUTE_NODE_TYPE_SAMPLE_PROCESSOR,
+    SAMPLE_PROCESSOR);
+
   /* add plugins */
   Track * tr;
   Plugin * pl;
@@ -1084,6 +1361,11 @@ graph_new (
       if (!tr->channel)
         continue;
 
+      /* add the fader */
+      graph_add_node ( \
+        self, ROUTE_NODE_TYPE_FADER,
+        &tr->channel->fader);
+
 #define ADD_PLUGIN \
           if (!pl || pl->deleting) \
             continue; \
@@ -1109,12 +1391,16 @@ graph_new (
           pl = tr->channel->plugins[j];
 
           ADD_PLUGIN;
+
+          plugin_update_latency (pl);
         }
       for (j = 0; j < tr->num_modulators; j++)
         {
           pl = tr->modulators[j]->plugin;
 
           ADD_PLUGIN;
+
+          plugin_update_latency (pl);
         }
     }
 #undef ADD_PLUGIN
@@ -1141,12 +1427,49 @@ graph_new (
    * now connect them
    * ======================== */
 
+  /* connect the sample processor */
+  node =
+    find_node_from_sample_processor (
+      self, SAMPLE_PROCESSOR);
+  port = SAMPLE_PROCESSOR->stereo_out->l;
+  node2 =
+    find_node_from_port (self, port);
+  node_connect (node, node2);
+  port = SAMPLE_PROCESSOR->stereo_out->r;
+  node2 =
+    find_node_from_port (self, port);
+  node_connect (node, node2);
+
+  Fader * fader;
   for (int i = 0; i < TRACKLIST->num_tracks; i++)
     {
       tr = TRACKLIST->tracks[i];
       if (!tr->channel)
         continue;
 
+      fader = &tr->channel->fader;
+
+      /* connect the fader */
+      node =
+        find_node_from_fader (
+          self, fader);
+      port = fader->stereo_in->l;
+      node2 =
+        find_node_from_port (self, port);
+      node_connect (node2, node);
+      port = fader->stereo_in->r;
+      node2 =
+        find_node_from_port (self, port);
+      node_connect (node2, node);
+      port = fader->stereo_out->l;
+      node2 =
+        find_node_from_port (self, port);
+      node_connect (node, node2);
+      port = fader->stereo_out->r;
+      node2 =
+        find_node_from_port (self, port);
+      node_connect (node, node2);
+
 #define CONNECT_PLUGIN \
           if (!pl || pl->deleting) \
             continue; \
@@ -1201,6 +1524,17 @@ graph_new (
         self, port);
     }
 
+  /* ========================
+   * calculate latencies of each port and each
+   * processor
+   * ======================== */
+
+  for (i = 0; i < self->terminal_node_count; i++)
+    {
+      set_node_playback_latency (
+        self->terminal_nodes[i], 0);
+    }
+
   g_message ("num trigger nodes %d",
              self->n_init_triggers);
   g_message ("num max trigger nodes %d",
