@@ -33,7 +33,7 @@
 #include "gui/widgets/main_window.h"
 #include "gui/widgets/midi_arranger.h"
 #include "gui/widgets/midi_modifier_arranger.h"
-#include "gui/widgets/midi_ruler.h"
+#include "gui/widgets/editor_ruler.h"
 #include "gui/widgets/timeline_arranger.h"
 #include "gui/widgets/timeline_ruler.h"
 #include "gui/widgets/top_bar.h"
@@ -87,13 +87,20 @@ transport_init (Transport * self,
       // set positions of playhead, start/end markers
       position_set_to_bar (&self->playhead_pos, 1);
       position_set_to_bar (&self->cue_pos, 1);
-      position_set_to_bar (&self->start_marker_pos, 1);
-      position_set_to_bar (&self->end_marker_pos, 128);
+      position_set_to_bar (
+        &self->start_marker_pos, 1);
+      position_set_to_bar (
+        &self->end_marker_pos, 128);
       position_set_to_bar (&self->loop_start_pos, 1);
-      position_set_to_bar (&self->loop_end_pos, 8);
+      position_set_to_bar (&self->loop_end_pos, 5);
 
       self->loop = 1;
 
+      self->metronome_enabled =
+        g_settings_get_int (
+          S_UI,
+          "metronome-enabled");
+
       transport_set_bpm (DEFAULT_BPM);
     }
 
@@ -155,6 +162,10 @@ transport_set_beat_unit (
     self->lticks_per_bar;
   self->sixteenths_per_beat =
     16.0 / (double) self->beat_unit;
+  g_warn_if_fail (self->ticks_per_bar > 0);
+  g_warn_if_fail (self->ticks_per_beat > 0);
+  g_warn_if_fail (self->lticks_per_bar > 0);
+  g_warn_if_fail (self->lticks_per_beat > 0);
 }
 
 void
@@ -197,18 +208,18 @@ transport_request_roll ()
 
 
 /**
- * Moves the playhead by the time corresponding to given samples.
+ * Moves the playhead by the time corresponding to
+ * given samples, taking into account the loop
+ * end point.
  */
-inline void
+void
 transport_add_to_playhead (
-  int frames)
+  Transport * self,
+  const int   frames)
 {
-  if (TRANSPORT->play_state == PLAYSTATE_ROLLING)
-    {
-      position_add_frames (&TRANSPORT->playhead_pos,
-                           frames);
-      EVENTS_PUSH (ET_PLAYHEAD_POS_CHANGED, NULL);
-    }
+  transport_position_add_frames (
+    self, &TRANSPORT->playhead_pos, frames);
+  EVENTS_PUSH (ET_PLAYHEAD_POS_CHANGED, NULL);
 }
 
 /**
@@ -221,6 +232,7 @@ transport_set_playhead_pos (
 {
   position_set_to_pos (
     &self->playhead_pos, pos);
+  EVENTS_PUSH (ET_PLAYHEAD_POS_CHANGED, NULL);
 }
 
 /**
@@ -272,7 +284,8 @@ transport_move_playhead (
             {
               region = lane->regions[l];
 
-              if (!region_is_hit (region, &PLAYHEAD))
+              if (!region_is_hit (
+                    region, PLAYHEAD->frames))
                 continue;
 
               for (j = 0;
@@ -282,7 +295,7 @@ transport_move_playhead (
                   midi_note = region->midi_notes[j];
 
                   if (midi_note_hit (
-                        midi_note, &PLAYHEAD))
+                        midi_note, PLAYHEAD->frames))
                     {
                       midi_events =
                         channel->piano_roll->
@@ -311,6 +324,17 @@ transport_move_playhead (
   EVENTS_PUSH (ET_PLAYHEAD_POS_CHANGED, NULL);
 }
 
+/**
+ * Sets whether metronome is enabled or not.
+ */
+void
+transport_set_metronome_enabled (
+  Transport * self,
+  const int   enabled)
+{
+  self->metronome_enabled = enabled;
+}
+
 /**
  * Updates the frames in all transport positions
  */
@@ -410,3 +434,51 @@ transport_set_loop (
 
   EVENTS_PUSH (ET_LOOP_TOGGLED, NULL);
 }
+
+/**
+ * Adds frames to the given global frames, while
+ * adjusting the new frames to loop back if the
+ * loop point was crossed.
+ *
+ * @return The new frames adjusted.
+ */
+long
+transport_frames_add_frames (
+  const Transport * self,
+  const long        gframes,
+  const int         frames)
+{
+  long new_frames = gframes + frames;
+
+  /* if start frames were before the loop-end point
+   * and the new frames are after (loop crossed) */
+  if (IS_TRANSPORT_LOOPING &&
+      gframes < self->loop_end_pos.frames &&
+      new_frames >= self->loop_end_pos.frames)
+    {
+      /* adjust the new frames */
+      new_frames +=
+        self->loop_start_pos.frames -
+        self->loop_end_pos.frames;
+    }
+
+  return new_frames;
+}
+
+/**
+ * Adds frames to the given position similar to
+ * position_add_frames(), except that it adjusts
+ * the new Position to loop back if the loop end
+ * point was crossed.
+ */
+void
+transport_position_add_frames (
+  const Transport * self,
+  Position *        pos,
+  const int         frames)
+{
+  position_from_frames (
+    pos,
+    transport_frames_add_frames (
+      self, pos->frames, frames));
+}
