@@ -33,15 +33,19 @@
 #include "gui/backend/piano_roll.h"
 #include "gui/widgets/arranger_playhead.h"
 #include "gui/widgets/audio_arranger.h"
-#include "gui/widgets/audio_clip_editor.h"
-#include "gui/widgets/audio_ruler.h"
+#include "gui/widgets/audio_editor_space.h"
+#include "gui/widgets/automation_arranger.h"
+#include "gui/widgets/automation_editor_space.h"
 #include "gui/widgets/automation_track.h"
 #include "gui/widgets/automation_tracklist.h"
 #include "gui/widgets/bot_dock_edge.h"
 #include "gui/widgets/center_dock.h"
 #include "gui/widgets/center_dock_bot_box.h"
 #include "gui/widgets/clip_editor.h"
+#include "gui/widgets/clip_editor_inner.h"
 #include "gui/widgets/channel.h"
+#include "gui/widgets/chord_arranger.h"
+#include "gui/widgets/chord_editor_space.h"
 #include "gui/widgets/color_area.h"
 #include "gui/widgets/header_notebook.h"
 #include "gui/widgets/home_toolbar.h"
@@ -51,10 +55,11 @@
 #include "gui/widgets/main_window.h"
 #include "gui/widgets/midi_arranger.h"
 #include "gui/widgets/midi_modifier_arranger.h"
-#include "gui/widgets/midi_ruler.h"
+#include "gui/widgets/editor_ruler.h"
 #include "gui/widgets/modulator_view.h"
+#include "gui/widgets/midi_editor_space.h"
 #include "gui/widgets/mixer.h"
-#include "gui/widgets/piano_roll_selection_info.h"
+#include "gui/widgets/editor_selection_info.h"
 #include "gui/widgets/pinned_tracklist.h"
 #include "gui/widgets/route_target_selector.h"
 #include "gui/widgets/ruler_marker.h"
@@ -101,57 +106,57 @@ on_playhead_changed ()
           gtk_widget_queue_allocate (
             GTK_WIDGET (MW_RULER));
         }
-      if (MW_PIANO_ROLL)
+      if (EDITOR_RULER)
         {
-          if (MIDI_RULER)
-            {
-              gtk_widget_queue_allocate (
-                GTK_WIDGET (MIDI_RULER));
-            }
-          if (MIDI_ARRANGER)
+          gtk_widget_queue_allocate (
+            GTK_WIDGET (EDITOR_RULER));
+        }
+      if (MW_MIDI_EDITOR_SPACE)
+        {
+          if (MW_MIDI_ARRANGER)
             {
               /*gtk_widget_queue_allocate (*/
                 /*GTK_WIDGET (MIDI_ARRANGER));*/
             }
-          if (MIDI_MODIFIER_ARRANGER)
+          if (MW_MIDI_MODIFIER_ARRANGER)
             {
               /*gtk_widget_queue_allocate (*/
                 /*GTK_WIDGET (MIDI_MODIFIER_ARRANGER));*/
             }
-          piano_roll_widget_refresh_labels (
-            MW_PIANO_ROLL, 0);
+          midi_editor_space_widget_refresh_labels (
+            MW_MIDI_EDITOR_SPACE, 0);
         }
-      if (MW_AUDIO_CLIP_EDITOR)
-        {
-          if (AUDIO_RULER)
-            gtk_widget_queue_allocate (
-              GTK_WIDGET (AUDIO_RULER));
-          if (AUDIO_ARRANGER)
-            {
-              /*ARRANGER_WIDGET_GET_PRIVATE (*/
-                /*AUDIO_ARRANGER);*/
-              /*g_object_ref (ar_prv->playhead);*/
-              /*gtk_container_remove (*/
-                /*AUDIO_ARRANGER,*/
-                /*ar_prv->playhead);*/
-              /*gtk_overlay_add_overlay (*/
-                /*AUDIO_ARRANGER,*/
-                /*ar_prv->playhead);*/
-              /*g_object_unref (ar_prv->playhead);*/
-              /*gtk_widget_queue_draw (*/
-                /*GTK_WIDGET (ar_prv->playhead));*/
-              /*gtk_widget_queue_allocate (*/
-                /*GTK_WIDGET (ar_prv->playhead));*/
+      /*if (MW_AUDIO_CLIP_EDITOR)*/
+        /*{*/
+          /*if (AUDIO_RULER)*/
             /*gtk_widget_queue_allocate (*/
-              /*GTK_WIDGET (AUDIO_ARRANGER));*/
-            /*arranger_widget_refresh (*/
-              /*Z_ARRANGER_WIDGET (AUDIO_ARRANGER));*/
-            /*gtk_widget_queue_resize (AUDIO_ARRANGER);*/
-            /*gtk_widget_queue_draw (AUDIO_ARRANGER);*/
-              gtk_widget_queue_allocate (
-                GTK_WIDGET (AUDIO_ARRANGER));
-            }
-        }
+              /*GTK_WIDGET (AUDIO_RULER));*/
+          /*if (AUDIO_ARRANGER)*/
+            /*{*/
+              /*[>ARRANGER_WIDGET_GET_PRIVATE (<]*/
+                /*[>AUDIO_ARRANGER);<]*/
+              /*[>g_object_ref (ar_prv->playhead);<]*/
+              /*[>gtk_container_remove (<]*/
+                /*[>AUDIO_ARRANGER,<]*/
+                /*[>ar_prv->playhead);<]*/
+              /*[>gtk_overlay_add_overlay (<]*/
+                /*[>AUDIO_ARRANGER,<]*/
+                /*[>ar_prv->playhead);<]*/
+              /*[>g_object_unref (ar_prv->playhead);<]*/
+              /*[>gtk_widget_queue_draw (<]*/
+                /*[>GTK_WIDGET (ar_prv->playhead));<]*/
+              /*[>gtk_widget_queue_allocate (<]*/
+                /*[>GTK_WIDGET (ar_prv->playhead));<]*/
+            /*[>gtk_widget_queue_allocate (<]*/
+              /*[>GTK_WIDGET (AUDIO_ARRANGER));<]*/
+            /*[>arranger_widget_refresh (<]*/
+              /*[>Z_ARRANGER_WIDGET (AUDIO_ARRANGER));<]*/
+            /*[>gtk_widget_queue_resize (AUDIO_ARRANGER);<]*/
+            /*[>gtk_widget_queue_draw (AUDIO_ARRANGER);<]*/
+              /*gtk_widget_queue_allocate (*/
+                /*GTK_WIDGET (AUDIO_ARRANGER));*/
+            /*}*/
+        /*}*/
     }
   /*aa = 1;*/
   return FALSE;
@@ -241,9 +246,9 @@ on_track_added (Track * track)
  * widths.
  */
 static int
-refresh_midi_ruler_and_arranger ()
+refresh_editor_ruler_and_arranger ()
 {
-  if (!ui_is_widget_revealed (MIDI_RULER))
+  if (!ui_is_widget_revealed (EDITOR_RULER))
     {
       g_usleep (1000);
       return TRUE;
@@ -251,40 +256,15 @@ refresh_midi_ruler_and_arranger ()
 
   /* ruler must be refreshed first to get the
    * correct px when calling ui_* functions */
-  midi_ruler_widget_refresh (
-    MIDI_RULER);
+  editor_ruler_widget_refresh (
+    EDITOR_RULER);
 
   /* remove all previous children and add new */
   arranger_widget_refresh (
-    Z_ARRANGER_WIDGET (MIDI_ARRANGER));
+    Z_ARRANGER_WIDGET (MW_MIDI_ARRANGER));
   arranger_widget_refresh (
     Z_ARRANGER_WIDGET (
-      MIDI_MODIFIER_ARRANGER));
-
-  CLIP_EDITOR->region_changed = 0;
-
-  return FALSE;
-}
-
-/**
- * Called when setting clip editor region using
- * g_idle_add to give the widgets time to have
- * widths.
- */
-static int
-refresh_audio_ruler_and_arranger ()
-{
-  if (!ui_is_widget_revealed (AUDIO_RULER))
-    {
-      g_usleep (1000);
-      return TRUE;
-    }
-
-  /* ruler must be refreshed first to get the
-   * correct px when calling ui_* functions */
-  audio_ruler_widget_refresh ();
-
-  /* remove all previous children and add new */
+      MW_MIDI_MODIFIER_ARRANGER));
 
   CLIP_EDITOR->region_changed = 0;
 
@@ -297,47 +277,20 @@ on_clip_editor_region_changed ()
   /* TODO */
   /*gtk_notebook_set_current_page (MAIN_WINDOW->bot_notebook, 0);*/
   Region * r = CLIP_EDITOR->region;
+  g_warn_if_fail (r);
 
   if (r)
     {
-      if (r->type == REGION_TYPE_MIDI)
-        {
-          gtk_stack_set_visible_child (
-            GTK_STACK (MW_CLIP_EDITOR),
-            GTK_WIDGET (MW_PIANO_ROLL_BOX));
+      gtk_stack_set_visible_child (
+        GTK_STACK (MW_CLIP_EDITOR),
+        GTK_WIDGET (MW_CLIP_EDITOR->main_box));
 
-          gtk_label_set_text (
-            MW_PIANO_ROLL->midi_name_label,
-            track_get_name (
-              r->lane->track));
+      clip_editor_inner_widget_refresh (
+        MW_CLIP_EDITOR_INNER);
 
-          color_area_widget_set_color (
-            MW_PIANO_ROLL->color_bar,
-            &r->lane->track->color);
-
-          g_idle_add (
-            refresh_midi_ruler_and_arranger,
-            NULL);
-        }
-      else if (r->type == REGION_TYPE_AUDIO)
-        {
-          gtk_stack_set_visible_child (
-            GTK_STACK (MW_CLIP_EDITOR),
-            GTK_WIDGET (MW_AUDIO_CLIP_EDITOR));
-
-          gtk_label_set_text (
-            MW_AUDIO_CLIP_EDITOR->track_name,
-            track_get_name (
-              r->lane->track));
-
-          color_area_widget_set_color (
-            MW_AUDIO_CLIP_EDITOR->color_bar,
-            &r->lane->track->color);
-
-          g_idle_add (
-            refresh_audio_ruler_and_arranger,
-            NULL);
-        }
+      g_idle_add (
+        refresh_editor_ruler_and_arranger,
+        NULL);
     }
   else
     {
@@ -346,7 +299,8 @@ on_clip_editor_region_changed ()
         GTK_WIDGET (
           MW_CLIP_EDITOR->no_selection_label));
     }
-  g_message ("clip editor region changed");
+
+  CLIP_EDITOR->region_cache = r;
 }
 
 static void
@@ -408,9 +362,9 @@ on_midi_note_selection_changed ()
       GTK_WIDGET (region->widget));
 
   gtk_widget_queue_allocate (
-    GTK_WIDGET (MIDI_MODIFIER_ARRANGER));
+    GTK_WIDGET (MW_MIDI_MODIFIER_ARRANGER));
 
-  piano_roll_selection_info_widget_refresh (
+  editor_selection_info_widget_refresh (
     MW_MAS_INFO, MA_SELECTIONS);
 }
 
@@ -719,14 +673,9 @@ events_process (void * data)
           break;
         case ET_REGION_CREATED:
         case ET_MARKER_CREATED:
-        case ET_AUTOMATION_POINT_CREATED:
-        case ET_AUTOMATION_CURVE_CREATED:
-        case ET_CHORD_OBJECT_CREATED:
         case ET_SCALE_OBJECT_CREATED:
-        case ET_CHORD_OBJECT_CHANGED:
         case ET_SCALE_OBJECT_CHANGED:
         case ET_REGION_REMOVED:
-        case ET_CHORD_OBJECT_REMOVED:
         case ET_SCALE_OBJECT_REMOVED:
         case ET_MARKER_REMOVED:
           /* FIXME track is passed so only
@@ -736,14 +685,21 @@ events_process (void * data)
           arranger_widget_refresh (
             Z_ARRANGER_WIDGET (MW_PINNED_TIMELINE));
           break;
+        case ET_CHORD_OBJECT_CREATED:
+        case ET_CHORD_OBJECT_CHANGED:
+        case ET_CHORD_OBJECT_REMOVED:
+          arranger_widget_refresh (
+            Z_ARRANGER_WIDGET (MW_CHORD_ARRANGER));
+          break;
         case ET_AUTOMATION_POINT_REMOVED:
         case ET_AUTOMATION_CURVE_REMOVED:
+        case ET_AUTOMATION_POINT_CREATED:
+        case ET_AUTOMATION_CURVE_CREATED:
           /* FIXME automation track is passed so
            * only refresh the automation lane */
           arranger_widget_refresh (
-            Z_ARRANGER_WIDGET (MW_TIMELINE));
-          arranger_widget_refresh (
-            Z_ARRANGER_WIDGET (MW_PINNED_TIMELINE));
+            Z_ARRANGER_WIDGET (
+              MW_AUTOMATION_ARRANGER));
           break;
         case ET_TL_SELECTIONS_CHANGED:
           inspector_widget_refresh (MW_INSPECTOR);
@@ -770,20 +726,25 @@ events_process (void * data)
             arranger_widget_refresh (
               Z_ARRANGER_WIDGET (
                 MW_TIMELINE));
-          else if (ev->arg == MIDI_RULER)
+          else if (ev->arg == EDITOR_RULER)
             {
-              arranger_widget_refresh (
-                Z_ARRANGER_WIDGET (
-                  MIDI_ARRANGER));
-              arranger_widget_refresh (
-                Z_ARRANGER_WIDGET (
-                  MIDI_MODIFIER_ARRANGER));
-            }
-          else if (ev->arg == AUDIO_RULER)
-            {
-              arranger_widget_refresh (
-                Z_ARRANGER_WIDGET (
-                  AUDIO_ARRANGER));
+              if (gtk_widget_get_visible (
+                    GTK_WIDGET (MW_MIDI_ARRANGER)))
+                {
+                  arranger_widget_refresh (
+                    Z_ARRANGER_WIDGET (
+                      MW_MIDI_ARRANGER));
+                  arranger_widget_refresh (
+                    Z_ARRANGER_WIDGET (
+                      MW_MIDI_MODIFIER_ARRANGER));
+                }
+              if (gtk_widget_get_visible (
+                    GTK_WIDGET (MW_AUDIO_ARRANGER)))
+                {
+                  arranger_widget_refresh (
+                    Z_ARRANGER_WIDGET (
+                      MW_AUDIO_ARRANGER));
+                }
             }
           break;
         case ET_CLIP_MARKER_POS_CHANGED:
@@ -796,6 +757,8 @@ events_process (void * data)
         case ET_TIMELINE_LOOP_MARKER_POS_CHANGED:
           gtk_widget_queue_allocate (
             GTK_WIDGET (MW_RULER));
+          gtk_widget_queue_draw (
+            GTK_WIDGET (EDITOR_RULER));
           break;
         case ET_TIMELINE_SONG_MARKER_POS_CHANGED:
           gtk_widget_queue_allocate (
@@ -829,23 +792,23 @@ events_process (void * data)
           toolbox_widget_refresh (MW_TOOLBOX);
           arranger_widget_refresh_cursor (
             Z_ARRANGER_WIDGET (MW_TIMELINE));
-          if (MIDI_ARRANGER &&
+          if (MW_MIDI_ARRANGER &&
               gtk_widget_get_realized (
-                GTK_WIDGET (MIDI_ARRANGER)))
+                GTK_WIDGET (MW_MIDI_ARRANGER)))
             arranger_widget_refresh_cursor (
-              Z_ARRANGER_WIDGET (MIDI_ARRANGER));
-          if (MIDI_MODIFIER_ARRANGER &&
+              Z_ARRANGER_WIDGET (MW_MIDI_ARRANGER));
+          if (MW_MIDI_MODIFIER_ARRANGER &&
               gtk_widget_get_realized (
-                GTK_WIDGET (MIDI_MODIFIER_ARRANGER)))
+                GTK_WIDGET (MW_MIDI_MODIFIER_ARRANGER)))
             arranger_widget_refresh_cursor (
               Z_ARRANGER_WIDGET (
-                MIDI_MODIFIER_ARRANGER));
+                MW_MIDI_MODIFIER_ARRANGER));
           break;
         case ET_TIME_SIGNATURE_CHANGED:
           ruler_widget_refresh (
             Z_RULER_WIDGET (MW_RULER));
           ruler_widget_refresh (
-            Z_RULER_WIDGET (MIDI_RULER));
+            Z_RULER_WIDGET (EDITOR_RULER));
           break;
         case ET_PLAYHEAD_POS_CHANGED:
           g_idle_add (on_playhead_changed,
@@ -863,7 +826,7 @@ events_process (void * data)
         case ET_TRACK_LANES_VISIBILITY_CHANGED:
           tracklist_widget_soft_refresh (
             MW_TRACKLIST);
-          timeline_arranger_widget_refresh_visibility (
+          timeline_arranger_widget_update_visibility (
             MW_TIMELINE);
           break;
         case ET_TRACK_ADDED:
@@ -912,7 +875,7 @@ events_process (void * data)
           ruler_widget_refresh (
             Z_RULER_WIDGET (MW_RULER));
           ruler_widget_refresh (
-            Z_RULER_WIDGET (MIDI_RULER));
+            Z_RULER_WIDGET (EDITOR_RULER));
           break;
         case ET_TRACK_STATE_CHANGED:
           on_track_state_changed (
@@ -935,10 +898,10 @@ events_process (void * data)
           /*arranger_widget_refresh (*/
             /*Z_ARRANGER_WIDGET (MIDI_ARRANGER));*/
           arranger_widget_refresh (
-            Z_ARRANGER_WIDGET (MIDI_ARRANGER));
+            Z_ARRANGER_WIDGET (MW_MIDI_ARRANGER));
           arranger_widget_refresh (
             Z_ARRANGER_WIDGET (
-              MIDI_MODIFIER_ARRANGER));
+              MW_MIDI_MODIFIER_ARRANGER));
           break;
         case ET_MIDI_NOTE_REMOVED:
           /*g_object_ref (((MidiNote *) ev->arg)->widget);*/
@@ -948,14 +911,14 @@ events_process (void * data)
           /*arranger_widget_refresh (*/
             /*Z_ARRANGER_WIDGET (MIDI_ARRANGER));*/
           midi_arranger_widget_refresh_children (
-            MIDI_ARRANGER);
+            MW_MIDI_ARRANGER);
           midi_modifier_arranger_widget_refresh_children (
-            MIDI_MODIFIER_ARRANGER);
+            MW_MIDI_MODIFIER_ARRANGER);
           break;
         case ET_PIANO_ROLL_HIGHLIGHTING_CHANGED:
-          if (MW_PIANO_ROLL)
-            piano_roll_widget_refresh_labels (
-              MW_PIANO_ROLL, 0);
+          if (MW_MIDI_EDITOR_SPACE)
+            midi_editor_space_widget_refresh_labels (
+              MW_MIDI_EDITOR_SPACE, 0);
           break;
         case ET_RULER_STATE_CHANGED:
           timeline_ruler_widget_refresh ();
@@ -994,7 +957,8 @@ events_process (void * data)
             ((Channel *)ev->arg)->widget);
           break;
         case ET_DRUM_MODE_CHANGED:
-          piano_roll_widget_refresh (MW_PIANO_ROLL);
+          midi_editor_space_widget_refresh (
+            MW_MIDI_EDITOR_SPACE);
           break;
         case ET_MODULATOR_ADDED:
           on_modulator_added ((Modulator *)ev->arg);
@@ -1011,7 +975,7 @@ events_process (void * data)
         case ET_TRACK_LANE_ADDED:
           tracklist_widget_soft_refresh (
             MW_TRACKLIST);
-          timeline_arranger_widget_refresh_visibility (
+          timeline_arranger_widget_update_visibility (
             MW_TIMELINE);
           break;
         case ET_LOOP_TOGGLED:
@@ -1019,7 +983,7 @@ events_process (void * data)
           gtk_widget_queue_draw (
             GTK_WIDGET (MW_RULER));
           gtk_widget_queue_draw (
-            GTK_WIDGET (MIDI_RULER));
+            GTK_WIDGET (EDITOR_RULER));
           break;
         case ET_MARKER_POSITIONS_CHANGED:
         case ET_CHORD_OBJECT_POSITIONS_CHANGED:
@@ -1030,9 +994,9 @@ events_process (void * data)
           /* redraw midi ruler if region
            * positions were changed */
           gtk_widget_queue_allocate (
-            GTK_WIDGET (MIDI_RULER));
+            GTK_WIDGET (EDITOR_RULER));
           gtk_widget_queue_draw (
-            GTK_WIDGET (MIDI_RULER));
+            GTK_WIDGET (EDITOR_RULER));
           break;
         case ET_TIMELINE_OBJECTS_IN_TRANSIT:
           timeline_arranger_widget_refresh_children (
@@ -1040,8 +1004,23 @@ events_process (void * data)
           timeline_arranger_widget_refresh_children (
             MW_PINNED_TIMELINE);
           if (TL_SELECTIONS->num_regions > 0)
-            midi_ruler_widget_refresh (
-              MIDI_RULER);
+            editor_ruler_widget_refresh (
+              EDITOR_RULER);
+          break;
+        case ET_CHORD_KEY_CHANGED:
+          for (int i = 0;
+               i < CHORD_EDITOR->num_chords; i++)
+            {
+              if (CHORD_EDITOR->chords[i] ==
+                (ChordDescriptor *) ev->arg)
+                {
+                  gtk_widget_queue_draw (
+                    GTK_WIDGET (
+                      MW_CHORD_EDITOR_SPACE->
+                        chord_keys[i]));
+                  break;
+                }
+            }
           break;
         default:
           g_message ("event not implemented yet");
