@@ -71,9 +71,7 @@ timeline_selections_init_loaded (
 
   _SET_OBJ (region);
   _SET_OBJ (marker);
-  _SET_OBJ (chord_object);
   _SET_OBJ (scale_object);
-  _SET_OBJ (automation_point);
 
 #undef _SET_OBJ
 }
@@ -87,10 +85,31 @@ timeline_selections_has_any (
 {
   return (
     ts->num_regions > 0 ||
-    ts->num_automation_points > 0 ||
     ts->num_scale_objects > 0 ||
-    ts->num_markers > 0 ||
-    ts->num_chord_objects > 0);
+    ts->num_markers > 0);
+}
+
+/**
+ * Code to run after deserializing.
+ *
+ * Used for copy-paste.
+ */
+void
+timeline_selections_post_deserialize (
+  TimelineSelections * ts)
+{
+  int i;
+
+#define _SET_OBJ(sc) \
+  for (i = 0; i < ts->num_##sc##s; i++) \
+    ts->sc##s[i]->obj_info.main =  \
+      ts->sc##s[i];
+
+  _SET_OBJ (region);
+  _SET_OBJ (marker);
+  _SET_OBJ (scale_object);
+
+#undef _SET_OBJ
 }
 
 /**
@@ -126,10 +145,8 @@ timeline_selections_set_cache_poses (
     }
 
   SET_CACHE_POS_W_LENGTH (Region, region);
-  SET_CACHE_POS (ChordObject, chord_object);
   SET_CACHE_POS (ScaleObject, scale_object);
   SET_CACHE_POS (Marker, marker);
-  SET_CACHE_POS (AutomationPoint, automation_point);
 
 #undef SET_CACHE_POS_W_LENGTH
 #undef SET_CACHE_POS
@@ -174,15 +191,10 @@ timeline_selections_reset_transient_poses (
 
   RESET_TRANS_POS_W_LENGTH (
     REGION, Region, region);
-  RESET_TRANS_POS (
-    CHORD_OBJECT, ChordObject, chord_object);
   RESET_TRANS_POS (
     SCALE_OBJECT, ScaleObject, scale_object);
   RESET_TRANS_POS (
     MARKER, Marker, marker);
-  RESET_TRANS_POS (
-    AUTOMATION_POINT, AutomationPoint,
-    automation_point);
 
 #undef RESET_TRANS_POS_W_LENGTH
 #undef RESET_TRANS_POS
@@ -212,12 +224,6 @@ timeline_selections_get_start_pos (
   ARRANGER_OBJ_SET_GIVEN_POS_TO (
     ts, Region, region, start_pos,
     transient, before, widget);
-  ARRANGER_OBJ_SET_GIVEN_POS_TO (
-    ts, AutomationPoint, automation_point, pos,
-    transient, before, widget);
-  ARRANGER_OBJ_SET_GIVEN_POS_TO (
-    ts, ChordObject, chord_object, pos,
-    transient, before, widget);
   ARRANGER_OBJ_SET_GIVEN_POS_TO (
     ts, ScaleObject, scale_object, pos,
     transient, before, widget);
@@ -249,12 +255,6 @@ timeline_selections_get_end_pos (
   ARRANGER_OBJ_SET_GIVEN_POS_TO (
     ts, Region, region, start_pos,
     transient, after, widget);
-  ARRANGER_OBJ_SET_GIVEN_POS_TO (
-    ts, AutomationPoint, automation_point, pos,
-    transient, after, widget);
-  ARRANGER_OBJ_SET_GIVEN_POS_TO (
-    ts, ChordObject, chord_object, pos,
-    transient, after, widget);
   ARRANGER_OBJ_SET_GIVEN_POS_TO (
     ts, ScaleObject, scale_object, pos,
     transient, after, widget);
@@ -284,12 +284,6 @@ timeline_selections_get_first_object (
   ARRANGER_OBJ_SET_GIVEN_POS_TO (
     ts, Region, region, start_pos,
     transient, before, widget);
-  ARRANGER_OBJ_SET_GIVEN_POS_TO (
-    ts, AutomationPoint, automation_point, pos,
-    transient, before, widget);
-  ARRANGER_OBJ_SET_GIVEN_POS_TO (
-    ts, ChordObject, chord_object, pos,
-    transient, before, widget);
   ARRANGER_OBJ_SET_GIVEN_POS_TO (
     ts, ScaleObject, scale_object, pos,
     transient, before, widget);
@@ -320,12 +314,6 @@ timeline_selections_get_last_object (
   ARRANGER_OBJ_SET_GIVEN_POS_TO (
     ts, Region, region, start_pos,
     transient, after, widget);
-  ARRANGER_OBJ_SET_GIVEN_POS_TO (
-    ts, AutomationPoint, automation_point, pos,
-    transient, after, widget);
-  ARRANGER_OBJ_SET_GIVEN_POS_TO (
-    ts, ChordObject, chord_object, pos,
-    transient, after, widget);
   ARRANGER_OBJ_SET_GIVEN_POS_TO (
     ts, ScaleObject, scale_object, pos,
     transient, after, widget);
@@ -370,19 +358,7 @@ timeline_selections_get_highest_track (
         region = ts->regions[i]->obj_info.main_trans;
       else
         region = ts->regions[i]->obj_info.main;
-      CHECK_POS (region->lane->track);
-    }
-  AutomationPoint * ap;
-  for (int i = 0; i < ts->num_automation_points; i++)
-    {
-      if (transient)
-        ap =
-          automation_point_get_main_trans_automation_point (
-                                                           ts->automation_points[i]);
-      else
-        ap = automation_point_get_main_automation_point (
-                                                           ts->automation_points[i]);
-      CHECK_POS (ap->at->track);
+      CHECK_POS (region_get_track (region));
     }
   CHECK_POS (P_CHORD_TRACK);
 
@@ -424,17 +400,7 @@ timeline_selections_get_lowest_track (
         region = ts->regions[i]->obj_info.main_trans;
       else
         region = ts->regions[i]->obj_info.main;
-      CHECK_POS (region->lane->track);
-    }
-  AutomationPoint * ap;
-  for (int i = 0; i < ts->num_automation_points; i++)
-    {
-      if (transient)
-        ap =
-          automation_point_get_main_trans_automation_point (ts->automation_points[i]);
-      else
-        ap = automation_point_get_main_automation_point (ts->automation_points[i]);
-      CHECK_POS (ap->at->track);
+      CHECK_POS (region_get_track (region));
     }
   CHECK_POS (P_CHORD_TRACK);
 
@@ -455,14 +421,9 @@ timeline_selections_get_lowest_track (
   }
 
 DEFINE_ADD_OBJECT (REGION, Region, region);
-DEFINE_ADD_OBJECT (
-  CHORD_OBJECT, ChordObject, chord_object);
 DEFINE_ADD_OBJECT (
   SCALE_OBJECT, ScaleObject, scale_object);
 DEFINE_ADD_OBJECT (MARKER, Marker, marker);
-DEFINE_ADD_OBJECT (
-  AUTOMATION_POINT, AutomationPoint,
-  automation_point);
 
 #undef DEFINE_ADD_OBJECT
 
@@ -476,14 +437,9 @@ DEFINE_ADD_OBJECT (
   }
 
 DEFINE_REMOVE_OBJ (REGION, Region, region);
-DEFINE_REMOVE_OBJ (
-  CHORD_OBJECT, ChordObject, chord_object);
 DEFINE_REMOVE_OBJ (
   SCALE_OBJECT, ScaleObject, scale_object);
 DEFINE_REMOVE_OBJ (MARKER, Marker, marker);
-DEFINE_REMOVE_OBJ (
-  AUTOMATION_POINT, AutomationPoint,
-  automation_point);
 
 #undef DEFINE_REMOVE_OBJ
 
@@ -499,14 +455,9 @@ DEFINE_REMOVE_OBJ (
   }
 
 DEFINE_CONTAINS_OBJ (REGION, Region, region);
-DEFINE_CONTAINS_OBJ (
-  CHORD_OBJECT, ChordObject, chord_object);
 DEFINE_CONTAINS_OBJ (
   SCALE_OBJECT, ScaleObject, scale_object);
 DEFINE_CONTAINS_OBJ (MARKER, Marker, marker);
-DEFINE_CONTAINS_OBJ (
-  AUTOMATION_POINT, AutomationPoint,
-  automation_point);
 
 #undef DEFINE_CONTAINS_OBJ
 
@@ -553,15 +504,10 @@ timeline_selections_set_to_transient_poses (
 
   SET_TO_TRANS_POS_W_LENGTH (
     REGION, Region, region);
-  SET_TO_TRANS_POS (
-    CHORD_OBJECT, ChordObject, chord_object);
   SET_TO_TRANS_POS (
     SCALE_OBJECT, ScaleObject, scale_object);
   SET_TO_TRANS_POS (
     MARKER, Marker, marker);
-  SET_TO_TRANS_POS (
-    AUTOMATION_POINT, AutomationPoint,
-    automation_point);
 
 #undef SET_TO_TRANS_POS_W_LENGTH
 #undef SET_TO_TRANS_POS
@@ -597,11 +543,6 @@ timeline_selections_clear (
 
   TL_REMOVE_OBJS (
     REGION, Region, region);
-  TL_REMOVE_OBJS (
-    AUTOMATION_POINT, AutomationPoint,
-    automation_point);
-  TL_REMOVE_OBJS (
-    CHORD_OBJECT, ChordObject, chord_object);
   TL_REMOVE_OBJS (
     SCALE_OBJECT, ScaleObject, scale_object);
   TL_REMOVE_OBJS (
@@ -614,13 +555,13 @@ timeline_selections_clear (
  * Clone the struct for copying, undoing, etc.
  */
 TimelineSelections *
-timeline_selections_clone ()
+timeline_selections_clone (
+  const TimelineSelections * src)
 {
   TimelineSelections * new_ts =
     calloc (1, sizeof (TimelineSelections));
 
   int i;
-  TimelineSelections * src = TL_SELECTIONS;
 
 #define TL_CLONE_OBJS(caps,cc,sc) \
   cc * sc, * new_##sc; \
@@ -636,11 +577,6 @@ timeline_selections_clone ()
 
   TL_CLONE_OBJS (
     REGION, Region, region);
-  TL_CLONE_OBJS (
-    AUTOMATION_POINT, AutomationPoint,
-    automation_point);
-  TL_CLONE_OBJS (
-    CHORD_OBJECT, ChordObject, chord_object);
   TL_CLONE_OBJS (
     SCALE_OBJECT, ScaleObject, scale_object);
   TL_CLONE_OBJS (
@@ -660,15 +596,6 @@ void
 timeline_selections_set_to_transient_values (
   TimelineSelections * ts)
 {
-  AutomationPoint * ap;
-  for (int i = 0; i < ts->num_automation_points;
-       i++)
-    {
-      ap = ts->automation_points[i];
-      ap->fvalue =
-        automation_point_get_main_trans_automation_point (
-          ap)->fvalue;
-    }
 }
 
 
@@ -703,10 +630,6 @@ timeline_selections_add_ticks (
 
   UPDATE_TL_POSES (
     Region, region);
-  UPDATE_TL_POSES (
-    AutomationPoint, automation_point);
-  UPDATE_TL_POSES (
-    ChordObject, chord_object);
   UPDATE_TL_POSES (
     ScaleObject, scale_object);
   UPDATE_TL_POSES (
@@ -720,6 +643,17 @@ timeline_selections_paste_to_pos (
   TimelineSelections * ts,
   Position *           pos)
 {
+  /* get selected track and check if valid for
+   * pasting */
+  Track * track =
+    TRACKLIST_SELECTIONS->tracks[0];
+  Track * orig_track =
+    TRACKLIST->tracks[ts->regions[0]->track_pos];
+  /* only allow lane regions for now */
+  if (ts->num_regions == 0 ||
+      orig_track->type != track->type)
+    return;
+
   int pos_ticks = position_to_ticks (pos);
 
   /* get pos of earliest object */
@@ -793,26 +727,9 @@ timeline_selections_paste_to_pos (
           region, REGION_CLONE_COPY_MAIN);
       region_print (cp);
       track_add_region (
-        cp->lane->track, cp, 0, F_GEN_NAME,
+        track, cp, 0, F_GEN_NAME,
         F_GEN_WIDGET);
     }
-  for (i = 0; i < ts->num_automation_points; i++)
-    {
-      AutomationPoint * ap =
-        ts->automation_points[i];
-
-      curr_ticks = position_to_ticks (&ap->pos);
-      position_from_ticks (&ap->pos,
-                           pos_ticks + DIFF);
-    }
-  for (i = 0; i < ts->num_chord_objects; i++)
-    {
-      ChordObject * chord = ts->chord_objects[i];
-
-      curr_ticks = position_to_ticks (&chord->pos);
-      position_from_ticks (&chord->pos,
-                           pos_ticks + DIFF);
-    }
   for (i = 0; i < ts->num_scale_objects; i++)
     {
       ScaleObject * scale = ts->scale_objects[i];
@@ -824,6 +741,25 @@ timeline_selections_paste_to_pos (
 #undef DIFF
 }
 
+void
+timeline_selections_free_full (
+  TimelineSelections * self)
+{
+  int i;
+  for (i = 0; i < self->num_regions; i++)
+    {
+      region_free (self->regions[i]);
+    }
+  for (i = 0; i < self->num_markers; i++)
+    {
+      marker_free (self->markers[i]);
+    }
+  for (i = 0; i < self->num_scale_objects; i++)
+    {
+      scale_object_free (self->scale_objects[i]);
+    }
+}
+
 void
 timeline_selections_free (
   TimelineSelections * self)
