@@ -21,7 +21,7 @@
 #include <math.h>
 
 #include "audio/position.h"
-#include "audio/quantize.h"
+#include "audio/quantize_options.h"
 #include "audio/snap_grid.h"
 #include "audio/transport.h"
 #include "gui/widgets/bot_dock_edge.h"
@@ -30,7 +30,7 @@
 #include "gui/widgets/main_window.h"
 #include "gui/widgets/midi_arranger.h"
 #include "gui/widgets/midi_modifier_arranger.h"
-#include "gui/widgets/midi_ruler.h"
+#include "gui/widgets/editor_ruler.h"
 #include "gui/widgets/ruler.h"
 #include "gui/widgets/timeline_arranger.h"
 #include "gui/widgets/timeline_ruler.h"
@@ -48,7 +48,8 @@ G_DEFINE_TYPE (DigitalMeterWidget,
                GTK_TYPE_DRAWING_AREA)
 
 #define FONT_SIZE 16
-#define SEG7_FONT "Segment7 Bold 16"
+/*#define SEG7_FONT "Segment7 Bold 16"*/
+#define SEG7_FONT "DSEG14 Classic Mini Italic 12"
 #define NORMAL_FONT "sans-serif Bold 11"
 #define CAPTION_FONT "sans-serif 7"
 #define SPACE_BETWEEN 6
@@ -219,15 +220,15 @@ draw_cb (
       if (bars < -100)
         text = g_strdup_printf ("%d", bars);
       else if (bars < -10)
-        text = g_strdup_printf (" %d", bars);
+        text = g_strdup_printf ("!%d", bars);
       else if (bars < 0)
-        text = g_strdup_printf ("  %d", bars);
+        text = g_strdup_printf ("!!%d", bars);
       else if (bars < 10)
-        text = g_strdup_printf ("   %d", bars);
+        text = g_strdup_printf ("!!!%d", bars);
       else if (bars < 100)
-        text = g_strdup_printf ("  %d", bars);
+        text = g_strdup_printf ("!!%d", bars);
       else
-        text = g_strdup_printf (" %d", bars);
+        text = g_strdup_printf ("!%d", bars);
       z_cairo_draw_text_full (
         cr, text, self->bars_start_pos,
         self->height_start_pos, SEG7_FONT);
@@ -577,10 +578,10 @@ drag_end (GtkGestureDrag *gesture,
         &PROJECT->snap_grid_timeline);
       snap_grid_update_snap_points (
         &PROJECT->snap_grid_midi);
-      quantize_update_snap_points (
-        &PROJECT->quantize_timeline);
-      quantize_update_snap_points (
-        &PROJECT->quantize_midi);
+      quantize_options_update_quantize_points (
+        QUANTIZE_OPTIONS_TIMELINE);
+      quantize_options_update_quantize_points (
+        QUANTIZE_OPTIONS_EDITOR);
     }
   self->update_note_length = 0;
   self->update_note_type = 0;
