@@ -33,13 +33,12 @@
 #include "gui/widgets/center_dock_bot_box.h"
 #include "gui/widgets/channel.h"
 #include "gui/widgets/clip_editor.h"
+#include "gui/widgets/editor_toolbar.h"
 #include "gui/widgets/file_browser.h"
 #include "gui/widgets/header_notebook.h"
 #include "gui/widgets/inspector.h"
 #include "gui/widgets/main_window.h"
 #include "gui/widgets/mixer.h"
-#include "gui/widgets/piano_roll.h"
-#include "gui/widgets/piano_roll_toolbar.h"
 #include "gui/widgets/plugin_browser.h"
 #include "gui/widgets/rack.h"
 #include "gui/widgets/ruler.h"
@@ -110,8 +109,8 @@ main_window_widget_refresh (MainWindowWidget * self)
   /* setup center dock */
   center_dock_widget_setup (MW_CENTER_DOCK);
 
-  piano_roll_toolbar_widget_setup (
-    MW_PIANO_ROLL_TOOLBAR);
+  editor_toolbar_widget_setup (
+    MW_EDITOR_TOOLBAR);
   timeline_toolbar_widget_setup (
     MW_TIMELINE_TOOLBAR);
 
@@ -119,8 +118,7 @@ main_window_widget_refresh (MainWindowWidget * self)
   if (MW_BOT_DOCK_EDGE && MW_CLIP_EDITOR)
     {
       clip_editor_widget_setup (
-        MW_CLIP_EDITOR,
-        &PROJECT->clip_editor);
+        MW_CLIP_EDITOR);
     }
 
   // set icons
@@ -199,80 +197,106 @@ main_window_widget_class_init (MainWindowWidgetClass * _klass)
                                  "main-window");
 }
 
-static GActionEntry actions[] = {
-
-  /* file menu */
-  { "new", activate_new },
-  { "open", activate_open },
-  { "save", activate_save },
-  { "save-as", activate_save_as },
-  { "export-as", activate_export_as },
-  { "properties", activate_properties },
-
-  /* edit menu */
-  { "undo", activate_undo },
-  { "redo", activate_redo },
-  { "cut", activate_cut },
-  { "copy", activate_copy },
-  { "paste", activate_paste },
-  { "delete", activate_delete },
-  { "duplicate", activate_duplicate },
-  { "clear-selection", activate_clear_selection },
-  { "select-all", activate_select_all },
-  /* selection submenu */
-  { "loop-selection", activate_loop_selection },
-
-  /* view menu */
-  { "toggle-left-panel", activate_toggle_left_panel },
-  { "toggle-right-panel", activate_toggle_right_panel },
-  { "toggle-bot-panel", activate_toggle_bot_panel },
-  { "toggle-top-panel", activate_toggle_top_panel },
-  { "toggle-status-bar", activate_toggle_status_bar },
-  { "zoom-in", activate_zoom_in },
-  { "zoom-out", activate_zoom_out },
-  { "original-size", activate_original_size },
-  { "best-fit", activate_best_fit },
-
-  /* snapping */
-  { "snap-to-grid", activate_snap_to_grid },
-  { "snap-keep-offset", activate_snap_keep_offset },
-  { "snap-events", activate_snap_events },
-
-  { "create-audio-track", activate_create_audio_track },
-  { "create-ins-track", activate_create_ins_track },
-  { "create-bus-track", activate_create_bus_track },
-  { "create-group-track", activate_create_group_track },
-  { "delete-selected-tracks", activate_delete_selected_tracks },
-
-  /* modes */
-  { "select-mode", activate_select_mode },
-  { "edit-mode", activate_edit_mode },
-  { "cut-mode", activate_cut_mode },
-  { "eraser-mode", activate_eraser_mode },
-  { "ramp-mode", activate_ramp_mode },
-  { "audition-mode", activate_audition_mode },
-
-  /* transport */
-  { "toggle-loop", NULL, NULL,
-    "true", change_state_loop },
-  { "goto-prev-marker", activate_goto_prev_marker },
-  { "goto-next-marker", activate_goto_next_marker },
-
-  /* tracks */
-  { "delete-selected-tracks", activate_delete_selected_tracks },
-  { "duplicate-selected-tracks", activate_duplicate_selected_tracks },
-
-  /* piano roll */
-  { "toggle-drum-mode", activate_toggle_drum_mode },
-
-  /* control room */
-  { "toggle-dim-output", NULL, NULL,
-    "true", change_state_dim_output },
-};
-
 static void
 main_window_widget_init (MainWindowWidget * self)
 {
+  GActionEntry actions[] = {
+
+    /* file menu */
+    { "new", activate_new },
+    { "open", activate_open },
+    { "save", activate_save },
+    { "save-as", activate_save_as },
+    { "export-as", activate_export_as },
+    { "properties", activate_properties },
+
+    /* edit menu */
+    { "undo", activate_undo },
+    { "redo", activate_redo },
+    { "cut", activate_cut },
+    { "copy", activate_copy },
+    { "paste", activate_paste },
+    { "delete", activate_delete },
+    { "duplicate", activate_duplicate },
+    { "clear-selection", activate_clear_selection },
+    { "select-all", activate_select_all },
+    /* selection submenu */
+    { "loop-selection", activate_loop_selection },
+
+    /* view menu */
+    { "toggle-left-panel",
+      activate_toggle_left_panel },
+    { "toggle-right-panel",
+      activate_toggle_right_panel },
+    { "toggle-bot-panel",
+      activate_toggle_bot_panel },
+    { "toggle-top-panel",
+      activate_toggle_top_panel },
+    { "toggle-status-bar",
+      activate_toggle_status_bar },
+    { "zoom-in", activate_zoom_in },
+    { "zoom-out", activate_zoom_out },
+    { "original-size", activate_original_size },
+    { "best-fit", activate_best_fit },
+
+    /* snapping, quantize */
+    { "snap-to-grid", activate_snap_to_grid },
+    { "snap-keep-offset",
+      activate_snap_keep_offset },
+    { "snap-events", activate_snap_events },
+    { "quick-quantize",
+      activate_quick_quantize, "s"},
+    { "quantize-options", activate_quantize_options,
+      "s" },
+
+    /* track actions */
+    { "create-audio-track",
+      activate_create_audio_track },
+    { "create-ins-track",
+      activate_create_ins_track },
+    { "create-bus-track",
+      activate_create_bus_track },
+    { "create-group-track",
+      activate_create_group_track },
+    { "delete-selected-tracks",
+      activate_delete_selected_tracks },
+
+    /* modes */
+    { "select-mode", activate_select_mode },
+    { "edit-mode", activate_edit_mode },
+    { "cut-mode", activate_cut_mode },
+    { "eraser-mode", activate_eraser_mode },
+    { "ramp-mode", activate_ramp_mode },
+    { "audition-mode", activate_audition_mode },
+
+    /* transport */
+    { "toggle-metronome", NULL, NULL,
+      g_settings_get_int (
+        S_UI, "metronome-enabled") ?
+        "true" : "false",
+      change_state_metronome },
+    { "toggle-loop", NULL, NULL,
+      "true", change_state_loop },
+    { "goto-prev-marker",
+      activate_goto_prev_marker },
+    { "goto-next-marker",
+      activate_goto_next_marker },
+
+    /* tracks */
+    { "delete-selected-tracks",
+      activate_delete_selected_tracks },
+    { "duplicate-selected-tracks",
+      activate_duplicate_selected_tracks },
+
+    /* piano roll */
+    { "toggle-drum-mode",
+      activate_toggle_drum_mode },
+
+    /* control room */
+    { "toggle-dim-output", NULL, NULL,
+      "true", change_state_dim_output },
+  };
+
   g_action_map_add_action_entries (
     G_ACTION_MAP (self),
     actions,
