@@ -37,14 +37,16 @@
 #include "gui/widgets/bot_dock_edge.h"
 #include "gui/widgets/center_dock.h"
 #include "gui/widgets/clip_editor.h"
+#include "gui/widgets/clip_editor_inner.h"
 #include "gui/widgets/color_area.h"
 #include "gui/widgets/inspector.h"
 #include "gui/widgets/main_window.h"
 #include "gui/widgets/midi_arranger_bg.h"
 #include "gui/widgets/midi_arranger.h"
+#include "gui/widgets/midi_editor_space.h"
 #include "gui/widgets/midi_modifier_arranger.h"
 #include "gui/widgets/midi_note.h"
-#include "gui/widgets/midi_ruler.h"
+#include "gui/widgets/editor_ruler.h"
 #include "gui/widgets/region.h"
 #include "gui/widgets/ruler.h"
 #include "gui/widgets/timeline_bg.h"
@@ -95,7 +97,7 @@ midi_arranger_widget_set_allocation (
         region->start_pos.total_ticks;
       Position tmp;
       int adj_px_per_key =
-        MW_PIANO_ROLL->px_per_key + 1;
+        MW_MIDI_EDITOR_SPACE->px_per_key + 1;
 
       /* use absolute position */
       position_from_ticks (
@@ -103,7 +105,7 @@ midi_arranger_widget_set_allocation (
         region_start_ticks +
         mn->start_pos.total_ticks);
       allocation->x =
-        ui_pos_to_px_piano_roll (
+        ui_pos_to_px_editor (
           &tmp, 1);
       allocation->y =
         adj_px_per_key *
@@ -124,7 +126,7 @@ midi_arranger_widget_set_allocation (
             region_start_ticks +
             mn->end_pos.total_ticks);
           allocation->width =
-            ui_pos_to_px_piano_roll (
+            ui_pos_to_px_editor (
               &tmp, 1) - allocation->x;
         }
     }
@@ -135,7 +137,7 @@ midi_arranger_widget_get_note_at_y (double y)
 {
   double adj_y = y - 1;
   double adj_px_per_key =
-    MW_PIANO_ROLL->px_per_key + 1;
+    MW_MIDI_EDITOR_SPACE->px_per_key + 1;
   if (PIANO_ROLL->drum_mode)
     {
       return
@@ -149,7 +151,7 @@ midi_arranger_widget_get_note_at_y (double y)
 }
 
 MidiNoteWidget *
-midi_arranger_widget_get_hit_midi_note (
+midi_arranger_widget_get_hit_note (
   MidiArrangerWidget *  self,
   double                x,
   double                y)
@@ -194,20 +196,21 @@ midi_arranger_widget_select_all (
 }
 
 static int
-on_motion (GtkWidget *      widget,
-           GdkEventMotion * event,
-           RegionWidget *   self)
+on_motion (
+  GtkWidget *      widget,
+  GdkEventMotion * event,
+  MidiArrangerWidget * self)
 {
   if (event->type == GDK_LEAVE_NOTIFY)
-    MIDI_ARRANGER->hovered_note = -1;
+    self->hovered_note = -1;
   else
-    MIDI_ARRANGER->hovered_note =
+    self->hovered_note =
       midi_arranger_widget_get_note_at_y (
         event->y);
   /*g_message ("hovered note: %d",*/
              /*MIDI_ARRANGER->hovered_note);*/
 
-  ARRANGER_WIDGET_GET_PRIVATE (MIDI_ARRANGER);
+  ARRANGER_WIDGET_GET_PRIVATE (self);
   gtk_widget_queue_draw (
               GTK_WIDGET (ar_prv->bg));
 
@@ -215,22 +218,22 @@ on_motion (GtkWidget *      widget,
 }
 
 void
-midi_arranger_widget_refresh_size (
+midi_arranger_widget_set_size (
   MidiArrangerWidget * self)
 {
   // set the size
-  RULER_WIDGET_GET_PRIVATE (MIDI_RULER);
+  RULER_WIDGET_GET_PRIVATE (EDITOR_RULER);
   gtk_widget_set_size_request (
     GTK_WIDGET (self),
     rw_prv->total_px,
-    MW_PIANO_ROLL->total_key_px);
+    MW_MIDI_EDITOR_SPACE->total_key_px);
 }
 
 void
 midi_arranger_widget_setup (
   MidiArrangerWidget * self)
 {
-  midi_arranger_widget_refresh_size (self);
+  midi_arranger_widget_set_size (self);
 
   ARRANGER_WIDGET_GET_PRIVATE (self);
   g_signal_connect (
@@ -261,7 +264,7 @@ midi_arranger_widget_get_cursor (
           tool == TOOL_EDIT)
         {
           MidiNoteWidget * mnw =
-            midi_arranger_widget_get_hit_midi_note (
+            midi_arranger_widget_get_hit_note (
               self,
               ar_prv->hover_x,
               ar_prv->hover_y);
@@ -358,7 +361,7 @@ midi_arranger_widget_show_context_menu (
   GtkMenuItem * menu_item;
 
   MidiNoteWidget * clicked_note =
-    midi_arranger_widget_get_hit_midi_note (
+    midi_arranger_widget_get_hit_note (
       self, x, y);
 
   if (clicked_note)
@@ -566,7 +569,9 @@ midi_arranger_widget_create_note (
 
   /* add it to region */
   midi_region_add_midi_note (
-    region, midi_note, F_GEN_WIDGET);
+    region, midi_note);
+
+  midi_note_gen_widget (midi_note);
 
   /* set visibility */
   arranger_object_info_set_widget_visibility_and_state (
@@ -1080,7 +1085,7 @@ midi_arranger_widget_on_drag_end (
   midi_arranger_widget_update_visibility (
     self);
   midi_modifier_arranger_widget_update_visibility (
-    MIDI_MODIFIER_ARRANGER);
+    MW_MIDI_MODIFIER_ARRANGER);
 
   self->start_midi_note = NULL;
   /*if (self->start_midi_note_clone)*/
