@@ -33,13 +33,6 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
-/**
- * \file
- *
- * Implementation of LV2 Plugin.
- */
-
-
 #define _POSIX_C_SOURCE 200809L /* for mkdtemp */
 #define _DARWIN_C_SOURCE        /* for mkdtemp on OSX */
 
@@ -166,7 +159,6 @@ _create_port(Lv2Plugin*   lv2_plugin,
   lv2_port->lilv_port =
     lilv_plugin_get_port_by_index (
       lv2_plugin->lilv_plugin, lv2_port_index);
-  lv2_port->sys_port  = NULL;
   const LilvNode* sym =
     lilv_port_get_symbol (
       lv2_plugin->lilv_plugin,
@@ -479,15 +471,17 @@ lv2_allocate_port_buffers(Lv2Plugin* plugin)
  * This function MUST set size and type appropriately.
  */
 static const void *
-lv2_get_port_value (const char * port_sym,
-                    void       * user_data,
-                    uint32_t   * size,
-                    uint32_t   * type)
+lv2_get_port_value (
+  const char * port_sym,
+  void       * user_data,
+  uint32_t   * size,
+  uint32_t   * type)
 {
   Lv2Plugin * lv2_plugin = (Lv2Plugin *) user_data;
 
-  Lv2Port * port = lv2_port_by_symbol (lv2_plugin,
-                                        port_sym);
+  Lv2Port * port =
+    lv2_plugin_get_lv2_port_by_symbol (
+      lv2_plugin, port_sym);
   *size = 0;
   *type = 0;
 
@@ -518,13 +512,16 @@ lv2_get_port_value (const char * port_sym,
 /*}*/
 
 /**
-   Get a port structure by symbol.
-
-   TODO: Build an index to make this faster, currently O(n) which may be
-   a problem when restoring the state of plugins with many ports.
-*/
+ * Returns the Lv2Port corresponding to the given
+ * symbol.
+ *
+ * TODO: Build an index to make this faster,
+ * currently O(n) which may be
+ * a problem when restoring the state of plugins
+ * with many ports.
+ */
 Lv2Port*
-lv2_port_by_symbol (
+lv2_plugin_get_lv2_port_by_symbol (
   Lv2Plugin* plugin,
   const char* sym)
 {
@@ -852,8 +849,14 @@ lv2_ui_write(SuilController controller,
   zix_ring_write(plugin->ui_events, buf, sizeof(buf));
 }
 
-void
-lv2_apply_ui_events(Lv2Plugin* plugin, uint32_t nframes)
+/**
+ * Read and apply control change events from UI,
+ * for plugins that have their own UIs.
+ */
+static void
+receive_ui_events (
+  Lv2Plugin* plugin,
+  uint32_t nframes)
 {
   if (!plugin->has_ui)
     {
@@ -886,8 +889,8 @@ lv2_apply_ui_events(Lv2Plugin* plugin, uint32_t nframes)
         {
           assert(ev.size == sizeof(float));
           port->control = *(float*)body;
-          /*g_message ("apply_ui_events %f",*/
-                     /*port->control);*/
+          g_message ("receiving UI event %f",
+                     port->control);
         }
       else if (ev.protocol ==
                PM_URIDS.atom_eventTransfer)
@@ -910,13 +913,26 @@ lv2_apply_ui_events(Lv2Plugin* plugin, uint32_t nframes)
     }
 }
 
+/**
+ * Returns the index of the Lv2Port corresponding to
+ * the given symbol.
+ *
+ * For LV2 UIs.
+ */
 uint32_t
-lv2_ui_port_index(SuilController controller, const char* symbol)
+lv2_ui_port_index (
+  SuilController controller,
+  const char* symbol)
 {
 	Lv2Plugin* const  plugin = (Lv2Plugin*)controller;
-	Lv2Port* port = lv2_port_by_symbol(plugin, symbol);
+	Lv2Port* port =
+    lv2_plugin_get_lv2_port_by_symbol (
+      plugin, symbol);
 
-	return port ? port->index : LV2UI_INVALID_PORT_INDEX;
+	return
+    port ?
+    port->index :
+    LV2UI_INVALID_PORT_INDEX;
 }
 
 void
@@ -991,31 +1007,45 @@ lv2_send_to_ui (
     }
 }
 
+/**
+ * Runs the plugin for this cycle.
+ */
 bool
 lv2_plugin_run (
-  Lv2Plugin* plugin, uint32_t nframes)
+  Lv2Plugin* plugin,
+  const int  nframes)
 {
-    /* Read and apply control change events from UI */
+  /* Read and apply control change events from UI */
   if (plugin->window)
-    lv2_apply_ui_events(plugin, nframes);
+    receive_ui_events (plugin, nframes);
 
   /* Run plugin for this cycle */
   lilv_instance_run(plugin->instance, nframes);
 
   /* Process any worker replies. */
-  lv2_worker_emit_responses(&plugin->state_worker, plugin->instance);
-  lv2_worker_emit_responses(&plugin->worker, plugin->instance);
+  lv2_worker_emit_responses (
+    &plugin->state_worker, plugin->instance);
+  lv2_worker_emit_responses (
+    &plugin->worker, plugin->instance);
 
-  /* Notify the plugin the run() cycle is finished */
-  if (plugin->worker.iface && plugin->worker.iface->end_run) {
-          plugin->worker.iface->end_run(plugin->instance->lv2_handle);
-  }
+  /* Notify the plugin the run() cycle is
+   * finished */
+  if (plugin->worker.iface &&
+      plugin->worker.iface->end_run)
+    {
+      plugin->worker.iface->end_run (
+        plugin->instance->lv2_handle);
+    }
 
   /* Check if it's time to send updates to the UI */
   plugin->event_delta_t += nframes;
-  bool     send_ui_updates = false;
-  uint32_t update_frames   = AUDIO_ENGINE->sample_rate / plugin->ui_update_hz;
-  if (plugin->has_ui && plugin->window && (plugin->event_delta_t > update_frames))
+  bool send_ui_updates = false;
+  uint32_t update_frames =
+    AUDIO_ENGINE->sample_rate /
+    plugin->ui_update_hz;
+  if (plugin->has_ui &&
+      plugin->window &&
+      (plugin->event_delta_t > update_frames))
     {
       send_ui_updates = true;
       plugin->event_delta_t = 0;
@@ -1219,12 +1249,29 @@ lv2_set_feature_data (Lv2Plugin * plugin)
   plugin->state_features[7] = NULL;
 }
 
+/**
+ * Returns the plugin's latency in samples.
+ *
+ * Searches for a port marked as reportsLatency
+ * and gets it value when a 0-size buffer is
+ * passed.
+ */
+long
+lv2_plugin_get_latency (
+  Lv2Plugin * pl)
+{
+  lv2_plugin_process (pl, PLAYHEAD->frames, 0);
+
+  return pl->plugin_latency;
+}
+
 /**
  * Returns a newly allocated plugin descriptor for the given LilvPlugin
  * if it can be hosted, otherwise NULL.
  */
 PluginDescriptor *
-lv2_create_descriptor_from_lilv (const LilvPlugin * lp)
+lv2_create_descriptor_from_lilv (
+  const LilvPlugin * lp)
 {
   const LilvNode*  lv2_uri =
     lilv_plugin_get_uri (lp);
@@ -1497,8 +1544,8 @@ lv2_free (Lv2Plugin * plugin)
  */
 int
 lv2_instantiate (
-  Lv2Plugin * self,
-  char *      preset_uri)
+  Lv2Plugin *  self,
+  char * preset_uri)
 {
   Plugin * plugin = self->plugin;
   int i;
@@ -1824,9 +1871,11 @@ lv2_instantiate (
   if (PM_LILV_NODES.opts.update_rate == 0.0)
     {
       /* Calculate a reasonable UI update frequency. */
-      /*self->ui_update_hz = (float)AUDIO_ENGINE->sample_rate /*/
+      /*self->ui_update_hz =*/
+        /*(float)AUDIO_ENGINE->sample_rate /*/
         /*AUDIO_ENGINE->midi_buf_size * 2.0f;*/
-      /*self->ui_update_hz = MAX(25.0f, self->ui_update_hz);*/
+      /*self->ui_update_hz =*/
+        /*MAX(25.0f, self->ui_update_hz);*/
 
       self->ui_update_hz =
         gdk_monitor_get_refresh_rate (
@@ -1840,14 +1889,19 @@ lv2_instantiate (
       /* Use user-specified UI update rate. */
       self->ui_update_hz =
         PM_LILV_NODES.opts.update_rate;
-      self->ui_update_hz = MAX(1.0f, self->ui_update_hz);
+      self->ui_update_hz =
+        MAX (1.0f, self->ui_update_hz);
     }
 
   /* The UI can only go so fast, clamp to reasonable limits */
-  self->ui_update_hz     = MIN(60, self->ui_update_hz);
-  PM_LILV_NODES.opts.buffer_size = MAX(4096, PM_LILV_NODES.opts.buffer_size);
-  g_message ("Comm buffers: %d bytes", PM_LILV_NODES.opts.buffer_size);
-  g_message ("Update rate:  %.01f Hz", self->ui_update_hz);
+  self->ui_update_hz =
+    MIN (60, self->ui_update_hz);
+  PM_LILV_NODES.opts.buffer_size =
+    MAX (4096, PM_LILV_NODES.opts.buffer_size);
+  g_message ("Comm buffers: %d bytes",
+             PM_LILV_NODES.opts.buffer_size);
+  g_message ("Update rate:  %.01f Hz",
+             self->ui_update_hz);
 
   static float f_samplerate = 0.f;
 
@@ -1989,27 +2043,43 @@ lv2_instantiate (
   return 0;
 }
 
+/**
+ * Processes the plugin for this cycle.
+ *
+ * @param g_start_frames The global start frames.
+ * @param nframes The number of frames to process.
+ */
 void
-lv2_plugin_process (Lv2Plugin * lv2_plugin)
+lv2_plugin_process (
+  Lv2Plugin * lv2_plugin,
+  const long  g_start_frames,
+  const int   nframes)
 {
-  int nframes = AUDIO_ENGINE->nframes;
 #ifdef HAVE_JACK
   jack_client_t * client = AUDIO_ENGINE->client;
 #endif
   int i, p;
 
-  /* If transport state is not as expected, then something has changed */
+  /* If transport state is not as expected, then
+   * something has changed */
   const bool xport_changed = (
-    lv2_plugin->rolling != (TRANSPORT->play_state == PLAYSTATE_ROLLING) ||
-    lv2_plugin->pos.frames != PLAYHEAD.frames ||
+    lv2_plugin->rolling !=
+      (IS_TRANSPORT_ROLLING) ||
+    /* FIXME don't save Position in Lv2Plugin.
+     * only safe long frames. */
+    lv2_plugin->gframes !=
+      g_start_frames ||
     lv2_plugin->bpm != TRANSPORT->bpm);
 
   uint8_t   pos_buf[256];
   LV2_Atom * lv2_pos = (LV2_Atom*) pos_buf;
   if (xport_changed)
     {
-      /* Build an LV2 position object to report change to
-       * plugin */
+      /* Build an LV2 position object to report
+       * change to plugin */
+      Position start_pos;
+      position_from_frames (
+        &start_pos, g_start_frames);
       lv2_atom_forge_set_buffer (
         &lv2_plugin->forge,
         pos_buf,
@@ -2026,7 +2096,7 @@ lv2_plugin_process (Lv2Plugin * lv2_plugin)
         PM_URIDS.time_frame);
       lv2_atom_forge_long (
         forge,
-        PLAYHEAD.frames);
+        g_start_frames);
       lv2_atom_forge_key (
         forge,
         PM_URIDS.time_speed);
@@ -2039,15 +2109,15 @@ lv2_plugin_process (Lv2Plugin * lv2_plugin)
         PM_URIDS.time_barBeat);
       lv2_atom_forge_float (
         forge,
-        (float) PLAYHEAD.beats - 1 +
-        ((float) PLAYHEAD.ticks /
+        (float) start_pos.beats - 1 +
+        ((float) start_pos.ticks /
           (float) TRANSPORT->ticks_per_beat));
       lv2_atom_forge_key (
         forge,
         PM_URIDS.time_bar);
       lv2_atom_forge_long (
         forge,
-        PLAYHEAD.bars - 1);
+        start_pos.bars - 1);
       lv2_atom_forge_key (
         forge,
         PM_URIDS.time_beatUnit);
@@ -2063,40 +2133,25 @@ lv2_plugin_process (Lv2Plugin * lv2_plugin)
       lv2_atom_forge_key (
         forge,
         PM_URIDS.time_beatsPerMinute);
-      lv2_atom_forge_float (forge, TRANSPORT->bpm);
+      lv2_atom_forge_float (
+        forge, TRANSPORT->bpm);
     }
 
-  /* Update transport state to expected values for next cycle */
-  lv2_plugin->pos.frames =
-    TRANSPORT->play_state == PLAYSTATE_ROLLING ?
-    PLAYHEAD.frames + nframes :
-    lv2_plugin->pos.frames;
-  lv2_plugin->bpm      = TRANSPORT->bpm;
-  lv2_plugin->rolling  = TRANSPORT->play_state == PLAYSTATE_ROLLING;
-
-    /*switch (TRANSPORT->play_state) {*/
-    /*case PLAYSTATE_PAUSE_REQUESTED:*/
-            /*[>jalv->play_state = JALV_PAUSED;<]*/
-            /*[>zix_sem_post(&jalv->paused);<]*/
-            /*break;*/
-    /*case JALV_PAUSED:*/
-      /*for (uint32_t p = 0; p < lv2_plugin->num_ports; ++p)*/
-        /*{*/
-          /*jack_port_t* jport = jalv->ports[p].sys_port;*/
-          /*if (jport && jalv->ports[p].flow == FLOW_OUTPUT)*/
-            /*{*/
-              /*void* buf = jack_port_get_buffer(jport, nframes);*/
-              /*if (jalv->ports[p].type == TYPE_EVENT) {*/
-                  /*jack_midi_clear_buffer(buf);*/
-              /*} else {*/
-                  /*memset(buf, '\0', nframes * sizeof(float));*/
-              /*}*/
-            /*}*/
-        /*}*/
-      /*return 0;*/
-    /*default:*/
-            /*break;*/
-    /*}*/
+  /* Update transport state to expected values for
+   * next cycle */
+  if (IS_TRANSPORT_ROLLING)
+    {
+      lv2_plugin->gframes =
+        transport_frames_add_frames (
+          TRANSPORT, lv2_plugin->gframes,
+          nframes);
+      lv2_plugin->rolling = 1;
+    }
+  else
+    {
+      lv2_plugin->rolling = 0;
+    }
+  lv2_plugin->bpm = TRANSPORT->bpm;
 
   /* Prepare port buffers */
   for (p = 0; p < lv2_plugin->num_ports; ++p)
@@ -2105,27 +2160,29 @@ lv2_plugin_process (Lv2Plugin * lv2_plugin)
       Port * port = lv2_port->port;
       if (port->identifier.type == TYPE_AUDIO)
         {
-          /* Connect lv2 ports  to plugin port buffers */
-          /*port->nframes = nframes;*/
+          /* connect lv2 ports to plugin port
+           * buffers */
           lilv_instance_connect_port (
             lv2_plugin->instance, p, port->buf);
         }
       else if (port->identifier.type == TYPE_CV)
         {
-          /* Connect plugin port directly to a
-           * CV buffer in the port.  according to
+          /* connect plugin port directly to a
+           * CV buffer in the port. according to
            * the docs it has the same size as an
            * audio port. */
           lilv_instance_connect_port (
             lv2_plugin->instance,
             p, port->buf);
         }
-      else if (port->identifier.type == TYPE_EVENT &&
-               port->identifier.flow == FLOW_INPUT)
+      else if (
+        port->identifier.type == TYPE_EVENT &&
+        port->identifier.flow == FLOW_INPUT)
         {
           lv2_evbuf_reset(lv2_port->evbuf, true);
 
-          /* Write transport change event if applicable */
+          /* Write transport change event if
+           * applicable */
           LV2_Evbuf_Iterator iter =
             lv2_evbuf_begin(lv2_port->evbuf);
           if (xport_changed)
@@ -2133,13 +2190,14 @@ lv2_plugin_process (Lv2Plugin * lv2_plugin)
               lv2_evbuf_write (
                 &iter, 0, 0,
                 lv2_pos->type, lv2_pos->size,
-                (const uint8_t*) LV2_ATOM_BODY(lv2_pos));
+                (const uint8_t*)
+                  LV2_ATOM_BODY (lv2_pos));
             }
 
           if (lv2_plugin->request_update)
             {
-              /* Plugin state has changed, request an
-               * update */
+              /* Plugin state has changed, request
+               * an update */
               const LV2_Atom_Object get = {
                       { sizeof(LV2_Atom_Object_Body),
                         PM_URIDS.atom_Object },
@@ -2147,7 +2205,8 @@ lv2_plugin_process (Lv2Plugin * lv2_plugin)
               lv2_evbuf_write (
                 &iter, 0, 0,
                 get.atom.type, get.atom.size,
-                (const uint8_t*) LV2_ATOM_BODY(&get));
+                (const uint8_t*)
+                  LV2_ATOM_BODY (&get));
             }
 
           if (port->midi_events->num_events > 0)
@@ -2159,12 +2218,12 @@ lv2_plugin_process (Lv2Plugin * lv2_plugin)
                 {
                   MidiEvent * ev =
                     &port->midi_events->events[i];
-                  /*g_message ("plugin event %d", i);*/
-                  /*g_message ("buf time %u %hhx %hhx %hhx",*/
-                             /*ev->time,*/
-                             /*ev->raw_buffer[0],*/
-                             /*ev->raw_buffer[1],*/
-                             /*ev->raw_buffer[2]);*/
+                  g_message ("plugin event %d", i);
+                  g_message ("buf time %u %hhx %hhx %hhx",
+                             ev->time,
+                             ev->raw_buffer[0],
+                             ev->raw_buffer[1],
+                             ev->raw_buffer[2]);
                   lv2_evbuf_write (
                     &iter,
                     ev->time, 0,
@@ -2196,12 +2255,13 @@ lv2_plugin_process (Lv2Plugin * lv2_plugin)
     lv2_plugin->plugin->ui_instantiated;
 
   /* Deliver MIDI output and UI events */
+  Port * port;
   for (p = 0; p < lv2_plugin->num_ports;
        ++p)
     {
       Lv2Port* const lv2_port =
         &lv2_plugin->ports[p];
-      Port * port = lv2_port->port;
+      port = lv2_port->port;
       if (port->identifier.flow == FLOW_OUTPUT &&
           port->identifier.type == TYPE_CONTROL &&
           lilv_port_has_property(
@@ -2259,8 +2319,14 @@ lv2_plugin_process (Lv2Plugin * lv2_plugin)
                   }
               }
           }
-        else if (send_ui_updates &&
-                 port->identifier.type == TYPE_CONTROL)
+        else if (
+          send_ui_updates &&
+          port->identifier.type == TYPE_CONTROL &&
+          lv2_port->automating)
+          /* don't write constrol UI events unless
+           * automating, otherwise this causes
+           * trembling of knobs while changing
+           * them */
           {
             char buf[sizeof(Lv2ControlChange) +
               sizeof(float)];
@@ -2272,6 +2338,7 @@ lv2_plugin_process (Lv2Plugin * lv2_plugin)
             *(float*)ev->body = lv2_port->control;
             /*g_message ("writing ui event %f",*/
                        /*lv2_port->control);*/
+            lv2_port->automating = 0;
             if (zix_ring_write (
                   lv2_plugin->plugin_events,
                   buf,
