@@ -22,7 +22,7 @@
 #include "actions/undo_manager.h"
 #include "audio/engine.h"
 #include "audio/mixer.h"
-#include "audio/quantize.h"
+#include "audio/quantize_options.h"
 #include "audio/track.h"
 #include "audio/tracklist.h"
 #include "gui/accel.h"
@@ -293,8 +293,10 @@ task_func (
       data->progress = 0.7;
       break;
     case TASK_END:
-      plugin_manager_scan_plugins (&ZRYTHM->plugin_manager);
-      file_manager_load_files (&ZRYTHM->file_manager);
+      plugin_manager_scan_plugins (
+        &ZRYTHM->plugin_manager);
+      file_manager_load_files (
+        &ZRYTHM->file_manager);
       data->message =
         _("Loading project");
       data->progress = 0.8;
@@ -691,6 +693,12 @@ zrythm_app_startup (GApplication* _app)
   accel_install_primary_action_accelerator (
     "KP_6",
     "win.goto-next-marker");
+  accel_install_primary_action_accelerator (
+    "Q",
+    "win.quick-quantize::global");
+  accel_install_primary_action_accelerator (
+    "<Alt>Q",
+    "win.quantize-options::global");
 }
 
 ZrythmApp *
@@ -698,8 +706,8 @@ zrythm_app_new ()
 {
   ZrythmApp * self =  g_object_new (
     ZRYTHM_APP_TYPE,
-    "application-id", "org.zrythm",
-    "resource-base-path", "/org/zrythm",
+    "application-id", "org.zrythm.Zrythm",
+    "resource-base-path", "/org/zrythm/Zrythm",
     "flags", G_APPLICATION_HANDLES_OPEN,
     NULL);
 
